@@ -1,3 +1,6 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: BUSL-1.1
+
 package ca
 
 import (
@@ -9,6 +12,7 @@ import (
 	"github.com/stretchr/testify/require"
 
 	"github.com/hashicorp/consul/agent/connect"
+	"github.com/hashicorp/consul/agent/consul/fsm"
 	"github.com/hashicorp/consul/agent/consul/state"
 	"github.com/hashicorp/consul/agent/structs"
 )
@@ -17,12 +21,22 @@ type consulCAMockDelegate struct {
 	state *state.Store
 }
 
-func (c *consulCAMockDelegate) State() *state.Store {
-	return c.state
+func (c *consulCAMockDelegate) ProviderState(id string) (*structs.CAConsulProviderState, error) {
+	_, s, err := c.state.CAProviderState(id)
+	return s, err
 }
 
 func (c *consulCAMockDelegate) ApplyCARequest(req *structs.CARequest) (interface{}, error) {
-	return ApplyCARequestToStore(c.state, req)
+	idx, _, err := c.state.CAConfig(nil)
+	if err != nil {
+		return nil, err
+	}
+
+	result := fsm.ApplyConnectCAOperationFromRequest(c.state, req, idx+1)
+	if err, ok := result.(error); ok && err != nil {
+		return nil, err
+	}
+	return result, nil
 }
 
 func newMockDelegate(t *testing.T, conf *structs.CAConfiguration) *consulCAMockDelegate {
@@ -43,8 +57,9 @@ func testConsulCAConfig() *structs.CAConfiguration {
 		Provider:  "consul",
 		Config: map[string]interface{}{
 			// Tests duration parsing after msgpack type mangling during raft apply.
-			"LeafCertTTL":         []uint8("72h"),
-			"IntermediateCertTTL": []uint8("288h"),
+			"LeafCertTTL":         []byte("72h"),
+			"IntermediateCertTTL": []byte("288h"),
+			"RootCertTTL":         []byte("87600h"),
 		},
 	}
 }
@@ -66,36 +81,41 @@ func requireNotEncoded(t *testing.T, v []byte) {
 func TestConsulCAProvider_Bootstrap(t *testing.T) {
 	t.Parallel()
 
-	require := require.New(t)
 	conf := testConsulCAConfig()
 	delegate := newMockDelegate(t, conf)
 
 	provider := TestConsulProvider(t, delegate)
-	require.NoError(provider.Configure(testProviderConfig(conf)))
-	require.NoError(provider.GenerateRoot())
+	require.NoError(t, provider.Configure(testProviderConfig(conf)))
 
-	root, err := provider.ActiveRoot()
-	require.NoError(err)
+	root, err := provider.GenerateCAChain()
+	require.NoError(t, err)
 
 	// Intermediate should be the same cert.
-	inter, err := provider.ActiveIntermediate()
-	require.NoError(err)
-	require.Equal(root, inter)
+	inter, err := provider.ActiveLeafSigningCert()
+	require.NoError(t, err)
+	require.Equal(t, root, inter)
 
 	// Should be a valid cert
 	parsed, err := connect.ParseCert(root)
-	require.NoError(err)
-	require.Equal(parsed.URIs[0].String(), fmt.Sprintf("spiffe://%s.consul", conf.ClusterID))
+	require.NoError(t, err)
+	require.Equal(t, parsed.URIs[0].String(), fmt.Sprintf("spiffe://%s.consul", conf.ClusterID))
 	requireNotEncoded(t, parsed.SubjectKeyId)
 	requireNotEncoded(t, parsed.AuthorityKeyId)
+
+	// test that the root cert ttl is the same as the expected value
+	// notice that we allow a margin of "error" of 10 minutes between the
+	// generateCA() creation and this check
+	defaultRootCertTTL, err := time.ParseDuration(structs.DefaultRootCertTTL)
+	require.NoError(t, err)
+	expectedNotAfter := time.Now().Add(defaultRootCertTTL).UTC()
+	require.WithinDuration(t, expectedNotAfter, parsed.NotAfter, 10*time.Minute, "expected parsed cert ttl to be the same as the value configured")
 }
 
 func TestConsulCAProvider_Bootstrap_WithCert(t *testing.T) {
 	t.Parallel()
 
 	// Make sure setting a custom private key/root cert works.
-	require := require.New(t)
-	rootCA := connect.TestCA(t, nil)
+	rootCA := connect.TestCAWithTTL(t, nil, 5*time.Hour)
 	conf := testConsulCAConfig()
 	conf.Config = map[string]interface{}{
 		"PrivateKey": rootCA.SigningKey,
@@ -104,12 +124,23 @@ func TestConsulCAProvider_Bootstrap_WithCert(t *testing.T) {
 	delegate := newMockDelegate(t, conf)
 
 	provider := TestConsulProvider(t, delegate)
-	require.NoError(provider.Configure(testProviderConfig(conf)))
-	require.NoError(provider.GenerateRoot())
+	require.NoError(t, provider.Configure(testProviderConfig(conf)))
 
-	root, err := provider.ActiveRoot()
-	require.NoError(err)
-	require.Equal(root, rootCA.RootCert)
+	root, err := provider.GenerateCAChain()
+	require.NoError(t, err)
+	require.Equal(t, root, rootCA.RootCert)
+
+	// Should be a valid cert
+	parsed, err := connect.ParseCert(root)
+	require.NoError(t, err)
+
+	// test that the default root cert ttl was not applied to the provided cert
+	defaultRootCertTTL, err := time.ParseDuration(structs.DefaultRootCertTTL)
+	require.NoError(t, err)
+	defaultNotAfter := time.Now().Add(defaultRootCertTTL).UTC()
+	// we can't compare given the "delta" between the time the cert is generated
+	// and when we start the test; so just look at the years for now, given different years
+	require.NotEqualf(t, defaultNotAfter.Year(), parsed.NotAfter.Year(), "parsed cert ttl expected to be different from default root cert ttl")
 }
 
 func TestConsulCAProvider_SignLeaf(t *testing.T) {
@@ -122,7 +153,6 @@ func TestConsulCAProvider_SignLeaf(t *testing.T) {
 	for _, tc := range KeyTestCases {
 		tc := tc
 		t.Run(tc.Desc, func(t *testing.T) {
-			require := require.New(t)
 			conf := testConsulCAConfig()
 			conf.Config["LeafCertTTL"] = "1h"
 			conf.Config["PrivateKeyType"] = tc.KeyType
@@ -130,8 +160,9 @@ func TestConsulCAProvider_SignLeaf(t *testing.T) {
 			delegate := newMockDelegate(t, conf)
 
 			provider := TestConsulProvider(t, delegate)
-			require.NoError(provider.Configure(testProviderConfig(conf)))
-			require.NoError(provider.GenerateRoot())
+			require.NoError(t, provider.Configure(testProviderConfig(conf)))
+			_, err := provider.GenerateCAChain()
+			require.NoError(t, err)
 
 			spiffeService := &connect.SpiffeIDService{
 				Host:       connect.TestClusterID + ".consul",
@@ -145,26 +176,26 @@ func TestConsulCAProvider_SignLeaf(t *testing.T) {
 				raw, _ := connect.TestCSR(t, spiffeService)
 
 				csr, err := connect.ParseCSR(raw)
-				require.NoError(err)
+				require.NoError(t, err)
 
 				cert, err := provider.Sign(csr)
-				require.NoError(err)
+				require.NoError(t, err)
 				requireTrailingNewline(t, cert)
 				parsed, err := connect.ParseCert(cert)
-				require.NoError(err)
-				require.Equal(spiffeService.URI(), parsed.URIs[0])
-				require.Equal(connect.ServiceCN("foo", "default", connect.TestClusterID), parsed.Subject.CommonName)
-				require.Equal(uint64(2), parsed.SerialNumber.Uint64())
+				require.NoError(t, err)
+				require.Equal(t, spiffeService.URI(), parsed.URIs[0])
+				require.Empty(t, parsed.Subject.CommonName)
+				require.Equal(t, uint64(3), parsed.SerialNumber.Uint64())
 				subjectKeyID, err := connect.KeyId(csr.PublicKey)
-				require.NoError(err)
-				require.Equal(subjectKeyID, parsed.SubjectKeyId)
+				require.NoError(t, err)
+				require.Equal(t, subjectKeyID, parsed.SubjectKeyId)
 				requireNotEncoded(t, parsed.SubjectKeyId)
 				requireNotEncoded(t, parsed.AuthorityKeyId)
 
 				// Ensure the cert is valid now and expires within the correct limit.
 				now := time.Now()
-				require.True(parsed.NotAfter.Sub(now) < time.Hour)
-				require.True(parsed.NotBefore.Before(now))
+				require.True(t, parsed.NotAfter.Sub(now) < time.Hour)
+				require.True(t, parsed.NotBefore.Before(now))
 			}
 
 			// Generate a new cert for another service and make sure
@@ -174,22 +205,22 @@ func TestConsulCAProvider_SignLeaf(t *testing.T) {
 				raw, _ := connect.TestCSR(t, spiffeService)
 
 				csr, err := connect.ParseCSR(raw)
-				require.NoError(err)
+				require.NoError(t, err)
 
 				cert, err := provider.Sign(csr)
-				require.NoError(err)
+				require.NoError(t, err)
 
 				parsed, err := connect.ParseCert(cert)
-				require.NoError(err)
-				require.Equal(spiffeService.URI(), parsed.URIs[0])
-				require.Equal(connect.ServiceCN("bar", "default", connect.TestClusterID), parsed.Subject.CommonName)
-				require.Equal(parsed.SerialNumber.Uint64(), uint64(2))
+				require.NoError(t, err)
+				require.Equal(t, spiffeService.URI(), parsed.URIs[0])
+				require.Empty(t, parsed.Subject.CommonName)
+				require.Equal(t, uint64(4), parsed.SerialNumber.Uint64())
 				requireNotEncoded(t, parsed.SubjectKeyId)
 				requireNotEncoded(t, parsed.AuthorityKeyId)
 
 				// Ensure the cert is valid now and expires within the correct limit.
-				require.True(time.Until(parsed.NotAfter) < 3*24*time.Hour)
-				require.True(parsed.NotBefore.Before(time.Now()))
+				require.True(t, time.Until(parsed.NotAfter) < 3*24*time.Hour)
+				require.True(t, parsed.NotBefore.Before(time.Now()))
 			}
 
 			spiffeAgent := &connect.SpiffeIDAgent{
@@ -202,23 +233,23 @@ func TestConsulCAProvider_SignLeaf(t *testing.T) {
 				raw, _ := connect.TestCSR(t, spiffeAgent)
 
 				csr, err := connect.ParseCSR(raw)
-				require.NoError(err)
+				require.NoError(t, err)
 
 				cert, err := provider.Sign(csr)
-				require.NoError(err)
+				require.NoError(t, err)
 
 				parsed, err := connect.ParseCert(cert)
-				require.NoError(err)
-				require.Equal(spiffeAgent.URI(), parsed.URIs[0])
-				require.Equal(connect.AgentCN("uuid", connect.TestClusterID), parsed.Subject.CommonName)
-				require.Equal(uint64(2), parsed.SerialNumber.Uint64())
+				require.NoError(t, err)
+				require.Equal(t, spiffeAgent.URI(), parsed.URIs[0])
+				require.Empty(t, parsed.Subject.CommonName)
+				require.Equal(t, uint64(5), parsed.SerialNumber.Uint64())
 				requireNotEncoded(t, parsed.SubjectKeyId)
 				requireNotEncoded(t, parsed.AuthorityKeyId)
 
 				// Ensure the cert is valid now and expires within the correct limit.
 				now := time.Now()
-				require.True(parsed.NotAfter.Sub(now) < time.Hour)
-				require.True(parsed.NotBefore.Before(now))
+				require.True(t, parsed.NotAfter.Sub(now) < time.Hour)
+				require.True(t, parsed.NotBefore.Before(now))
 			}
 		})
 	}
@@ -236,15 +267,15 @@ func TestConsulCAProvider_CrossSignCA(t *testing.T) {
 	for _, tc := range tests {
 		tc := tc
 		t.Run(tc.Desc, func(t *testing.T) {
-			require := require.New(t)
 
 			conf1 := testConsulCAConfig()
 			delegate1 := newMockDelegate(t, conf1)
 			provider1 := TestConsulProvider(t, delegate1)
 			conf1.Config["PrivateKeyType"] = tc.SigningKeyType
 			conf1.Config["PrivateKeyBits"] = tc.SigningKeyBits
-			require.NoError(provider1.Configure(testProviderConfig(conf1)))
-			require.NoError(provider1.GenerateRoot())
+			require.NoError(t, provider1.Configure(testProviderConfig(conf1)))
+			_, err := provider1.GenerateCAChain()
+			require.NoError(t, err)
 
 			conf2 := testConsulCAConfig()
 			conf2.CreateIndex = 10
@@ -252,8 +283,9 @@ func TestConsulCAProvider_CrossSignCA(t *testing.T) {
 			provider2 := TestConsulProvider(t, delegate2)
 			conf2.Config["PrivateKeyType"] = tc.CSRKeyType
 			conf2.Config["PrivateKeyBits"] = tc.CSRKeyBits
-			require.NoError(provider2.Configure(testProviderConfig(conf2)))
-			require.NoError(provider2.GenerateRoot())
+			require.NoError(t, provider2.Configure(testProviderConfig(conf2)))
+			_, err = provider2.GenerateCAChain()
+			require.NoError(t, err)
 
 			testCrossSignProviders(t, provider1, provider2)
 		})
@@ -261,52 +293,52 @@ func TestConsulCAProvider_CrossSignCA(t *testing.T) {
 }
 
 func testCrossSignProviders(t *testing.T, provider1, provider2 Provider) {
-	require := require.New(t)
 
 	// Get the root from the new provider to be cross-signed.
-	newRootPEM, err := provider2.ActiveRoot()
-	require.NoError(err)
-	newRoot, err := connect.ParseCert(newRootPEM)
-	require.NoError(err)
+	root, err := provider2.GenerateCAChain()
+	require.NoError(t, err)
+
+	newRoot, err := connect.ParseCert(root)
+	require.NoError(t, err)
 	oldSubject := newRoot.Subject.CommonName
 	requireNotEncoded(t, newRoot.SubjectKeyId)
 	requireNotEncoded(t, newRoot.AuthorityKeyId)
 
-	newInterPEM, err := provider2.ActiveIntermediate()
-	require.NoError(err)
+	newInterPEM, err := provider2.ActiveLeafSigningCert()
+	require.NoError(t, err)
 	newIntermediate, err := connect.ParseCert(newInterPEM)
-	require.NoError(err)
+	require.NoError(t, err)
 	requireNotEncoded(t, newIntermediate.SubjectKeyId)
 	requireNotEncoded(t, newIntermediate.AuthorityKeyId)
 
 	// Have provider1 cross sign our new root cert.
 	xcPEM, err := provider1.CrossSignCA(newRoot)
-	require.NoError(err)
+	require.NoError(t, err)
 	xc, err := connect.ParseCert(xcPEM)
-	require.NoError(err)
+	require.NoError(t, err)
 	requireNotEncoded(t, xc.SubjectKeyId)
 	requireNotEncoded(t, xc.AuthorityKeyId)
 
-	oldRootPEM, err := provider1.ActiveRoot()
-	require.NoError(err)
-	oldRoot, err := connect.ParseCert(oldRootPEM)
-	require.NoError(err)
+	p1Root, err := provider1.GenerateCAChain()
+	require.NoError(t, err)
+	oldRoot, err := connect.ParseCert(p1Root)
+	require.NoError(t, err)
 	requireNotEncoded(t, oldRoot.SubjectKeyId)
 	requireNotEncoded(t, oldRoot.AuthorityKeyId)
 
 	// AuthorityKeyID should now be the signing root's, SubjectKeyId should be kept.
-	require.Equal(oldRoot.SubjectKeyId, xc.AuthorityKeyId,
+	require.Equal(t, oldRoot.SubjectKeyId, xc.AuthorityKeyId,
 		"newSKID=%x\nnewAKID=%x\noldSKID=%x\noldAKID=%x\nxcSKID=%x\nxcAKID=%x",
 		newRoot.SubjectKeyId, newRoot.AuthorityKeyId,
 		oldRoot.SubjectKeyId, oldRoot.AuthorityKeyId,
 		xc.SubjectKeyId, xc.AuthorityKeyId)
-	require.Equal(newRoot.SubjectKeyId, xc.SubjectKeyId)
+	require.Equal(t, newRoot.SubjectKeyId, xc.SubjectKeyId)
 
 	// Subject name should not have changed.
-	require.Equal(oldSubject, xc.Subject.CommonName)
+	require.Equal(t, oldSubject, xc.Subject.CommonName)
 
 	// Issuer should be the signing root.
-	require.Equal(oldRoot.Issuer.CommonName, xc.Issuer.CommonName)
+	require.Equal(t, oldRoot.Issuer.CommonName, xc.Issuer.CommonName)
 
 	// Get a leaf cert so we can verify against the cross-signed cert.
 	spiffeService := &connect.SpiffeIDService{
@@ -318,13 +350,13 @@ func testCrossSignProviders(t *testing.T, provider1, provider2 Provider) {
 	raw, _ := connect.TestCSR(t, spiffeService)
 
 	leafCsr, err := connect.ParseCSR(raw)
-	require.NoError(err)
+	require.NoError(t, err)
 
 	leafPEM, err := provider2.Sign(leafCsr)
-	require.NoError(err)
+	require.NoError(t, err)
 
 	cert, err := connect.ParseCert(leafPEM)
-	require.NoError(err)
+	require.NoError(t, err)
 	requireNotEncoded(t, cert.SubjectKeyId)
 	requireNotEncoded(t, cert.AuthorityKeyId)
 
@@ -342,10 +374,34 @@ func testCrossSignProviders(t *testing.T, provider1, provider2 Provider) {
 			Intermediates: intermediatePool,
 			Roots:         rootPool,
 		})
-		require.NoError(err)
+		require.NoError(t, err)
 	}
 }
 
+func testCrossSignProvidersShouldFail(t *testing.T, provider1, provider2 Provider) {
+	t.Helper()
+
+	// Get the root from the new provider to be cross-signed.
+	root, err := provider2.GenerateCAChain()
+	require.NoError(t, err)
+
+	newRoot, err := connect.ParseCert(root)
+	require.NoError(t, err)
+	requireNotEncoded(t, newRoot.SubjectKeyId)
+	requireNotEncoded(t, newRoot.AuthorityKeyId)
+
+	newInterPEM, err := provider2.ActiveLeafSigningCert()
+	require.NoError(t, err)
+	newIntermediate, err := connect.ParseCert(newInterPEM)
+	require.NoError(t, err)
+	requireNotEncoded(t, newIntermediate.SubjectKeyId)
+	requireNotEncoded(t, newIntermediate.AuthorityKeyId)
+
+	// Have provider1 cross sign our new root cert.
+	_, err = provider1.CrossSignCA(newRoot)
+	require.Error(t, err)
+}
+
 func TestConsulProvider_SignIntermediate(t *testing.T) {
 	if testing.Short() {
 		t.Skip("too slow for testing.Short")
@@ -358,15 +414,15 @@ func TestConsulProvider_SignIntermediate(t *testing.T) {
 	for _, tc := range tests {
 		tc := tc
 		t.Run(tc.Desc, func(t *testing.T) {
-			require := require.New(t)
 
 			conf1 := testConsulCAConfig()
 			delegate1 := newMockDelegate(t, conf1)
 			provider1 := TestConsulProvider(t, delegate1)
 			conf1.Config["PrivateKeyType"] = tc.SigningKeyType
 			conf1.Config["PrivateKeyBits"] = tc.SigningKeyBits
-			require.NoError(provider1.Configure(testProviderConfig(conf1)))
-			require.NoError(provider1.GenerateRoot())
+			require.NoError(t, provider1.Configure(testProviderConfig(conf1)))
+			_, err := provider1.GenerateCAChain()
+			require.NoError(t, err)
 
 			conf2 := testConsulCAConfig()
 			conf2.CreateIndex = 10
@@ -377,7 +433,7 @@ func TestConsulProvider_SignIntermediate(t *testing.T) {
 			cfg := testProviderConfig(conf2)
 			cfg.IsPrimary = false
 			cfg.Datacenter = "dc2"
-			require.NoError(provider2.Configure(cfg))
+			require.NoError(t, provider2.Configure(cfg))
 
 			testSignIntermediateCrossDC(t, provider1, provider2)
 		})
@@ -386,22 +442,22 @@ func TestConsulProvider_SignIntermediate(t *testing.T) {
 }
 
 func testSignIntermediateCrossDC(t *testing.T, provider1, provider2 Provider) {
-	require := require.New(t)
 
 	// Get the intermediate CSR from provider2.
-	csrPEM, err := provider2.GenerateIntermediateCSR()
-	require.NoError(err)
+	csrPEM, opaque, err := provider2.GenerateIntermediateCSR()
+	require.NoError(t, err)
 	csr, err := connect.ParseCSR(csrPEM)
-	require.NoError(err)
+	require.NoError(t, err)
 
 	// Sign the CSR with provider1.
 	intermediatePEM, err := provider1.SignIntermediate(csr)
-	require.NoError(err)
-	rootPEM, err := provider1.ActiveRoot()
-	require.NoError(err)
+	require.NoError(t, err)
+	root, err := provider1.GenerateCAChain()
+	require.NoError(t, err)
+	rootPEM := root
 
 	// Give the new intermediate to provider2 to use.
-	require.NoError(provider2.SetIntermediate(intermediatePEM, rootPEM))
+	require.NoError(t, provider2.SetIntermediate(intermediatePEM, rootPEM, opaque))
 
 	// Have provider2 sign a leaf cert and make sure the chain is correct.
 	spiffeService := &connect.SpiffeIDService{
@@ -413,13 +469,13 @@ func testSignIntermediateCrossDC(t *testing.T, provider1, provider2 Provider) {
 	raw, _ := connect.TestCSR(t, spiffeService)
 
 	leafCsr, err := connect.ParseCSR(raw)
-	require.NoError(err)
+	require.NoError(t, err)
 
 	leafPEM, err := provider2.Sign(leafCsr)
-	require.NoError(err)
+	require.NoError(t, err)
 
 	cert, err := connect.ParseCert(leafPEM)
-	require.NoError(err)
+	require.NoError(t, err)
 	requireNotEncoded(t, cert.SubjectKeyId)
 	requireNotEncoded(t, cert.AuthorityKeyId)
 
@@ -434,34 +490,50 @@ func testSignIntermediateCrossDC(t *testing.T, provider1, provider2 Provider) {
 		Intermediates: intermediatePool,
 		Roots:         rootPool,
 	})
-	require.NoError(err)
+	require.NoError(t, err)
 }
 
 func TestConsulCAProvider_MigrateOldID(t *testing.T) {
-	t.Parallel()
-
-	require := require.New(t)
-	conf := testConsulCAConfig()
-	delegate := newMockDelegate(t, conf)
-
-	// Create an entry with an old-style ID.
-	_, err := delegate.ApplyCARequest(&structs.CARequest{
-		Op: structs.CAOpSetProviderState,
-		ProviderState: &structs.CAConsulProviderState{
-			ID: ",",
+	cases := []struct {
+		name  string
+		oldID string
+	}{
+		{
+			name:  "original-unhashed",
+			oldID: ",",
 		},
-	})
-	require.NoError(err)
-	_, providerState, err := delegate.state.CAProviderState(",")
-	require.NoError(err)
-	require.NotNil(providerState)
+		{
+			name:  "hash-v1",
+			oldID: hexStringHash(",,true"),
+		},
+	}
 
-	provider := TestConsulProvider(t, delegate)
-	require.NoError(provider.Configure(testProviderConfig(conf)))
-	require.NoError(provider.GenerateRoot())
+	for _, tc := range cases {
+		t.Run(tc.name, func(t *testing.T) {
+			conf := testConsulCAConfig()
+			delegate := newMockDelegate(t, conf)
 
-	// After running Configure, the old ID entry should be gone.
-	_, providerState, err = delegate.state.CAProviderState(",")
-	require.NoError(err)
-	require.Nil(providerState)
+			// Create an entry with an old-style ID.
+			_, err := delegate.ApplyCARequest(&structs.CARequest{
+				Op: structs.CAOpSetProviderState,
+				ProviderState: &structs.CAConsulProviderState{
+					ID: tc.oldID,
+				},
+			})
+			require.NoError(t, err)
+			_, providerState, err := delegate.state.CAProviderState(tc.oldID)
+			require.NoError(t, err)
+			require.NotNil(t, providerState)
+
+			provider := TestConsulProvider(t, delegate)
+			require.NoError(t, provider.Configure(testProviderConfig(conf)))
+			_, err = provider.GenerateCAChain()
+			require.NoError(t, err)
+
+			// After running Configure, the old ID entry should be gone.
+			_, providerState, err = delegate.state.CAProviderState(tc.oldID)
+			require.NoError(t, err)
+			require.Nil(t, providerState)
+		})
+	}
 }
