@@ -1,3 +1,6 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: BUSL-1.1
+
 package consul
 
 import (
@@ -9,7 +12,7 @@ import (
 	"testing"
 	"time"
 
-	msgpackrpc "github.com/hashicorp/net-rpc-msgpackrpc"
+	msgpackrpc "github.com/hashicorp/consul-net-rpc/net-rpc-msgpackrpc"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 
@@ -17,6 +20,7 @@ import (
 	"github.com/hashicorp/consul/agent/connect"
 	ca "github.com/hashicorp/consul/agent/connect/ca"
 	"github.com/hashicorp/consul/agent/structs"
+	"github.com/hashicorp/consul/sdk/testutil"
 	"github.com/hashicorp/consul/sdk/testutil/retry"
 	"github.com/hashicorp/consul/testrpc"
 )
@@ -37,8 +41,6 @@ func TestConnectCARoots(t *testing.T) {
 
 	t.Parallel()
 
-	assert := assert.New(t)
-	require := require.New(t)
 	dir1, s1 := testServer(t)
 	defer os.RemoveAll(dir1)
 	defer s1.Shutdown()
@@ -53,29 +55,29 @@ func TestConnectCARoots(t *testing.T) {
 	ca2 := connect.TestCA(t, nil)
 	ca2.Active = false
 	idx, _, err := state.CARoots(nil)
-	require.NoError(err)
+	require.NoError(t, err)
 	ok, err := state.CARootSetCAS(idx, idx, []*structs.CARoot{ca1, ca2})
-	assert.True(ok)
-	require.NoError(err)
+	assert.True(t, ok)
+	require.NoError(t, err)
 	_, caCfg, err := state.CAConfig(nil)
-	require.NoError(err)
+	require.NoError(t, err)
 
 	// Request
 	args := &structs.DCSpecificRequest{
 		Datacenter: "dc1",
 	}
 	var reply structs.IndexedCARoots
-	require.NoError(msgpackrpc.CallWithCodec(codec, "ConnectCA.Roots", args, &reply))
+	require.NoError(t, msgpackrpc.CallWithCodec(codec, "ConnectCA.Roots", args, &reply))
 
 	// Verify
-	assert.Equal(ca1.ID, reply.ActiveRootID)
-	assert.Len(reply.Roots, 2)
+	assert.Equal(t, ca1.ID, reply.ActiveRootID)
+	assert.Len(t, reply.Roots, 2)
 	for _, r := range reply.Roots {
 		// These must never be set, for security
-		assert.Equal("", r.SigningCert)
-		assert.Equal("", r.SigningKey)
+		assert.Equal(t, "", r.SigningCert)
+		assert.Equal(t, "", r.SigningKey)
 	}
-	assert.Equal(fmt.Sprintf("%s.consul", caCfg.ClusterID), reply.TrustDomain)
+	assert.Equal(t, fmt.Sprintf("%s.consul", caCfg.ClusterID), reply.TrustDomain)
 }
 
 func TestConnectCAConfig_GetSet(t *testing.T) {
@@ -85,7 +87,6 @@ func TestConnectCAConfig_GetSet(t *testing.T) {
 
 	t.Parallel()
 
-	assert := assert.New(t)
 	dir1, s1 := testServer(t)
 	defer os.RemoveAll(dir1)
 	defer s1.Shutdown()
@@ -100,14 +101,14 @@ func TestConnectCAConfig_GetSet(t *testing.T) {
 			Datacenter: "dc1",
 		}
 		var reply structs.CAConfiguration
-		assert.NoError(msgpackrpc.CallWithCodec(codec, "ConnectCA.ConfigurationGet", args, &reply))
+		assert.NoError(t, msgpackrpc.CallWithCodec(codec, "ConnectCA.ConfigurationGet", args, &reply))
 
 		actual, err := ca.ParseConsulCAConfig(reply.Config)
-		assert.NoError(err)
+		assert.NoError(t, err)
 		expected, err := ca.ParseConsulCAConfig(s1.config.CAConfig.Config)
-		assert.NoError(err)
-		assert.Equal(reply.Provider, s1.config.CAConfig.Provider)
-		assert.Equal(actual, expected)
+		assert.NoError(t, err)
+		assert.Equal(t, reply.Provider, s1.config.CAConfig.Provider)
+		assert.Equal(t, actual, expected)
 	}
 
 	testState := map[string]string{"foo": "bar"}
@@ -140,15 +141,15 @@ func TestConnectCAConfig_GetSet(t *testing.T) {
 			Datacenter: "dc1",
 		}
 		var reply structs.CAConfiguration
-		assert.NoError(msgpackrpc.CallWithCodec(codec, "ConnectCA.ConfigurationGet", args, &reply))
+		assert.NoError(t, msgpackrpc.CallWithCodec(codec, "ConnectCA.ConfigurationGet", args, &reply))
 
 		actual, err := ca.ParseConsulCAConfig(reply.Config)
-		assert.NoError(err)
+		assert.NoError(t, err)
 		expected, err := ca.ParseConsulCAConfig(newConfig.Config)
-		assert.NoError(err)
-		assert.Equal(reply.Provider, newConfig.Provider)
-		assert.Equal(actual, expected)
-		assert.Equal(testState, reply.State)
+		assert.NoError(t, err)
+		assert.Equal(t, reply.Provider, newConfig.Provider)
+		assert.Equal(t, actual, expected)
+		assert.Equal(t, testState, reply.State)
 	}
 }
 
@@ -160,10 +161,10 @@ func TestConnectCAConfig_GetSet_ACLDeny(t *testing.T) {
 	t.Parallel()
 
 	dir1, s1 := testServerWithConfig(t, func(c *Config) {
-		c.ACLDatacenter = "dc1"
+		c.PrimaryDatacenter = "dc1"
 		c.ACLsEnabled = true
-		c.ACLMasterToken = TestDefaultMasterToken
-		c.ACLDefaultPolicy = "deny"
+		c.ACLInitialManagementToken = TestDefaultInitialManagementToken
+		c.ACLResolverSettings.ACLDefaultPolicy = "deny"
 	})
 	defer os.RemoveAll(dir1)
 	defer s1.Shutdown()
@@ -174,11 +175,11 @@ func TestConnectCAConfig_GetSet_ACLDeny(t *testing.T) {
 	testrpc.WaitForLeader(t, s1.RPC, "dc1")
 
 	opReadToken, err := upsertTestTokenWithPolicyRules(
-		codec, TestDefaultMasterToken, "dc1", `operator = "read"`)
+		codec, TestDefaultInitialManagementToken, "dc1", `operator = "read"`)
 	require.NoError(t, err)
 
 	opWriteToken, err := upsertTestTokenWithPolicyRules(
-		codec, TestDefaultMasterToken, "dc1", `operator = "write"`)
+		codec, TestDefaultInitialManagementToken, "dc1", `operator = "write"`)
 	require.NoError(t, err)
 
 	// Update a config value
@@ -214,7 +215,7 @@ pY0heYeK9A6iOLrzqxSerkXXQyj5e9bE4VgUnxgPU6g=
 	args := &structs.CARequest{
 		Datacenter:   "dc1",
 		Config:       newConfig,
-		WriteRequest: structs.WriteRequest{Token: TestDefaultMasterToken},
+		WriteRequest: structs.WriteRequest{Token: TestDefaultInitialManagementToken},
 	}
 	var reply interface{}
 	require.NoError(t, msgpackrpc.CallWithCodec(codec, "ConnectCA.ConfigurationSet", args, &reply))
@@ -253,7 +254,6 @@ func TestConnectCAConfig_GetSetForceNoCrossSigning(t *testing.T) {
 
 	t.Parallel()
 
-	require := require.New(t)
 	// Setup a server with a built-in CA that as artificially disabled cross
 	// signing. This is simpler than running tests with external CA dependencies.
 	dir1, s1 := testServerWithConfig(t, func(c *Config) {
@@ -271,8 +271,8 @@ func TestConnectCAConfig_GetSetForceNoCrossSigning(t *testing.T) {
 		Datacenter: "dc1",
 	}
 	var rootList structs.IndexedCARoots
-	require.NoError(msgpackrpc.CallWithCodec(codec, "ConnectCA.Roots", rootReq, &rootList))
-	require.Len(rootList.Roots, 1)
+	require.NoError(t, msgpackrpc.CallWithCodec(codec, "ConnectCA.Roots", rootReq, &rootList))
+	require.Len(t, rootList.Roots, 1)
 	oldRoot := rootList.Roots[0]
 
 	// Get the starting config
@@ -281,20 +281,20 @@ func TestConnectCAConfig_GetSetForceNoCrossSigning(t *testing.T) {
 			Datacenter: "dc1",
 		}
 		var reply structs.CAConfiguration
-		require.NoError(msgpackrpc.CallWithCodec(codec, "ConnectCA.ConfigurationGet", args, &reply))
+		require.NoError(t, msgpackrpc.CallWithCodec(codec, "ConnectCA.ConfigurationGet", args, &reply))
 
 		actual, err := ca.ParseConsulCAConfig(reply.Config)
-		require.NoError(err)
+		require.NoError(t, err)
 		expected, err := ca.ParseConsulCAConfig(s1.config.CAConfig.Config)
-		require.NoError(err)
-		require.Equal(reply.Provider, s1.config.CAConfig.Provider)
-		require.Equal(actual, expected)
+		require.NoError(t, err)
+		require.Equal(t, reply.Provider, s1.config.CAConfig.Provider)
+		require.Equal(t, actual, expected)
 	}
 
 	// Update to a new CA with different key. This should fail since the existing
 	// CA doesn't support cross signing so can't rotate safely.
 	_, newKey, err := connect.GeneratePrivateKey()
-	require.NoError(err)
+	require.NoError(t, err)
 	newConfig := &structs.CAConfiguration{
 		Provider: "consul",
 		Config: map[string]interface{}{
@@ -308,7 +308,7 @@ func TestConnectCAConfig_GetSetForceNoCrossSigning(t *testing.T) {
 		}
 		var reply interface{}
 		err := msgpackrpc.CallWithCodec(codec, "ConnectCA.ConfigurationSet", args, &reply)
-		require.EqualError(err, "The current CA Provider does not support cross-signing. "+
+		require.EqualError(t, err, "The current CA Provider does not support cross-signing. "+
 			"You can try again with ForceWithoutCrossSigningSet but this may cause disruption"+
 			" - see documentation for more.")
 	}
@@ -322,7 +322,7 @@ func TestConnectCAConfig_GetSetForceNoCrossSigning(t *testing.T) {
 		}
 		var reply interface{}
 		err := msgpackrpc.CallWithCodec(codec, "ConnectCA.ConfigurationSet", args, &reply)
-		require.NoError(err)
+		require.NoError(t, err)
 	}
 
 	// Make sure the new root has been added but with no cross-signed intermediate
@@ -331,23 +331,23 @@ func TestConnectCAConfig_GetSetForceNoCrossSigning(t *testing.T) {
 			Datacenter: "dc1",
 		}
 		var reply structs.IndexedCARoots
-		require.NoError(msgpackrpc.CallWithCodec(codec, "ConnectCA.Roots", args, &reply))
-		require.Len(reply.Roots, 2)
+		require.NoError(t, msgpackrpc.CallWithCodec(codec, "ConnectCA.Roots", args, &reply))
+		require.Len(t, reply.Roots, 2)
 
 		for _, r := range reply.Roots {
 			if r.ID == oldRoot.ID {
 				// The old root should no longer be marked as the active root,
 				// and none of its other fields should have changed.
-				require.False(r.Active)
-				require.Equal(r.Name, oldRoot.Name)
-				require.Equal(r.RootCert, oldRoot.RootCert)
-				require.Equal(r.SigningCert, oldRoot.SigningCert)
-				require.Equal(r.IntermediateCerts, oldRoot.IntermediateCerts)
+				require.False(t, r.Active)
+				require.Equal(t, r.Name, oldRoot.Name)
+				require.Equal(t, r.RootCert, oldRoot.RootCert)
+				require.Equal(t, r.SigningCert, oldRoot.SigningCert)
+				require.Equal(t, r.IntermediateCerts, oldRoot.IntermediateCerts)
 			} else {
 				// The new root should NOT have a valid cross-signed cert from the old
 				// root as an intermediate.
-				require.True(r.Active)
-				require.Empty(r.IntermediateCerts)
+				require.True(t, r.Active)
+				require.Empty(t, r.IntermediateCerts)
 			}
 		}
 	}
@@ -360,154 +360,308 @@ func TestConnectCAConfig_TriggerRotation(t *testing.T) {
 
 	t.Parallel()
 
-	assert := assert.New(t)
-	require := require.New(t)
-	dir1, s1 := testServer(t)
-	defer os.RemoveAll(dir1)
-	defer s1.Shutdown()
-	codec := rpcClient(t, s1)
-	defer codec.Close()
+	cases := []struct {
+		name     string
+		configFn func() (*structs.CAConfiguration, error)
+	}{
+		{
+			name: "new private key provided",
+			configFn: func() (*structs.CAConfiguration, error) {
+				// Update the provider config to use a new private key, which should
+				// cause a rotation.
+				_, newKey, err := connect.GeneratePrivateKey()
+				if err != nil {
+					return nil, err
+				}
 
-	testrpc.WaitForTestAgent(t, s1.RPC, "dc1")
-
-	// Store the current root
-	rootReq := &structs.DCSpecificRequest{
-		Datacenter: "dc1",
-	}
-	var rootList structs.IndexedCARoots
-	require.Nil(msgpackrpc.CallWithCodec(codec, "ConnectCA.Roots", rootReq, &rootList))
-	assert.Len(rootList.Roots, 1)
-	oldRoot := rootList.Roots[0]
-
-	// Update the provider config to use a new private key, which should
-	// cause a rotation.
-	_, newKey, err := connect.GeneratePrivateKey()
-	assert.NoError(err)
-	newConfig := &structs.CAConfiguration{
-		Provider: "consul",
-		Config: map[string]interface{}{
-			"PrivateKey": newKey,
-			"RootCert":   "",
+				return &structs.CAConfiguration{
+					Provider: "consul",
+					Config: map[string]interface{}{
+						"PrivateKey": newKey,
+						"RootCert":   "",
+					},
+				}, nil
+			},
+		},
+		{
+			name: "update private key bits",
+			configFn: func() (*structs.CAConfiguration, error) {
+				return &structs.CAConfiguration{
+					Provider: "consul",
+					Config: map[string]interface{}{
+						"PrivateKeyType": "ec",
+						"PrivateKeyBits": 384,
+					},
+				}, nil
+			},
+		},
+		{
+			name: "update private key type",
+			configFn: func() (*structs.CAConfiguration, error) {
+				return &structs.CAConfiguration{
+					Provider: "consul",
+					Config: map[string]interface{}{
+						"PrivateKeyType": "rsa",
+						"PrivateKeyBits": "2048",
+					},
+				}, nil
+			},
 		},
 	}
-	{
-		args := &structs.CARequest{
-			Datacenter: "dc1",
-			Config:     newConfig,
-		}
-		var reply interface{}
 
-		require.NoError(msgpackrpc.CallWithCodec(codec, "ConnectCA.ConfigurationSet", args, &reply))
-	}
+	for _, tc := range cases {
+		t.Run(tc.name, func(t *testing.T) {
+			dir1, s1 := testServer(t)
+			defer os.RemoveAll(dir1)
+			defer s1.Shutdown()
+			codec := rpcClient(t, s1)
+			defer codec.Close()
 
-	// Make sure the new root has been added along with an intermediate
-	// cross-signed by the old root.
-	var newRootPEM string
-	{
-		args := &structs.DCSpecificRequest{
-			Datacenter: "dc1",
-		}
-		var reply structs.IndexedCARoots
-		require.Nil(msgpackrpc.CallWithCodec(codec, "ConnectCA.Roots", args, &reply))
-		assert.Len(reply.Roots, 2)
+			testrpc.WaitForTestAgent(t, s1.RPC, "dc1")
 
-		for _, r := range reply.Roots {
-			if r.ID == oldRoot.ID {
-				// The old root should no longer be marked as the active root,
-				// and none of its other fields should have changed.
-				assert.False(r.Active)
-				assert.Equal(r.Name, oldRoot.Name)
-				assert.Equal(r.RootCert, oldRoot.RootCert)
-				assert.Equal(r.SigningCert, oldRoot.SigningCert)
-				assert.Equal(r.IntermediateCerts, oldRoot.IntermediateCerts)
-			} else {
-				newRootPEM = r.RootCert
-				// The new root should have a valid cross-signed cert from the old
-				// root as an intermediate.
-				assert.True(r.Active)
-				assert.Len(r.IntermediateCerts, 1)
-
-				xc := testParseCert(t, r.IntermediateCerts[0])
-				oldRootCert := testParseCert(t, oldRoot.RootCert)
-				newRootCert := testParseCert(t, r.RootCert)
-
-				// Should have the authority key ID and signature algo of the
-				// (old) signing CA.
-				assert.Equal(xc.AuthorityKeyId, oldRootCert.AuthorityKeyId)
-				assert.NotEqual(xc.SubjectKeyId, oldRootCert.SubjectKeyId)
-				assert.Equal(xc.SignatureAlgorithm, oldRootCert.SignatureAlgorithm)
-
-				// The common name and SAN should not have changed.
-				assert.Equal(xc.Subject.CommonName, newRootCert.Subject.CommonName)
-				assert.Equal(xc.URIs, newRootCert.URIs)
+			// Store the current root
+			rootReq := &structs.DCSpecificRequest{
+				Datacenter: "dc1",
 			}
+			var rootList structs.IndexedCARoots
+			require.Nil(t, msgpackrpc.CallWithCodec(codec, "ConnectCA.Roots", rootReq, &rootList))
+			assert.Len(t, rootList.Roots, 1)
+			oldRoot := rootList.Roots[0]
+
+			newConfig, err := tc.configFn()
+			require.NoError(t, err)
+
+			{
+				args := &structs.CARequest{
+					Datacenter: "dc1",
+					Config:     newConfig,
+				}
+				var reply interface{}
+
+				require.NoError(t, msgpackrpc.CallWithCodec(codec, "ConnectCA.ConfigurationSet", args, &reply))
+			}
+
+			// Make sure the new root has been added along with an intermediate
+			// cross-signed by the old root.
+			var newRootPEM string
+			testutil.RunStep(t, "ensure roots look correct", func(t *testing.T) {
+				args := &structs.DCSpecificRequest{
+					Datacenter: "dc1",
+				}
+				var reply structs.IndexedCARoots
+				require.Nil(t, msgpackrpc.CallWithCodec(codec, "ConnectCA.Roots", args, &reply))
+				assert.Len(t, reply.Roots, 2)
+
+				for _, r := range reply.Roots {
+					if r.ID == oldRoot.ID {
+						// The old root should no longer be marked as the active root,
+						// and none of its other fields should have changed.
+						assert.False(t, r.Active)
+						assert.Equal(t, r.Name, oldRoot.Name)
+						assert.Equal(t, r.RootCert, oldRoot.RootCert)
+						assert.Equal(t, r.SigningCert, oldRoot.SigningCert)
+						assert.Equal(t, r.IntermediateCerts, oldRoot.IntermediateCerts)
+					} else {
+						newRootPEM = r.RootCert
+						// The new root should have a valid cross-signed cert from the old
+						// root as an intermediate.
+						assert.True(t, r.Active)
+						assert.Len(t, r.IntermediateCerts, 1)
+
+						xc := testParseCert(t, r.IntermediateCerts[0])
+						oldRootCert := testParseCert(t, oldRoot.RootCert)
+						newRootCert := testParseCert(t, r.RootCert)
+
+						// Should have the authority key ID and signature algo of the
+						// (old) signing CA.
+						assert.Equal(t, xc.AuthorityKeyId, oldRootCert.AuthorityKeyId)
+						assert.NotEqual(t, xc.SubjectKeyId, oldRootCert.SubjectKeyId)
+						assert.Equal(t, xc.SignatureAlgorithm, oldRootCert.SignatureAlgorithm)
+
+						// The common name and SAN should not have changed.
+						assert.Equal(t, xc.Subject.CommonName, newRootCert.Subject.CommonName)
+						assert.Equal(t, xc.URIs, newRootCert.URIs)
+					}
+				}
+			})
+
+			testutil.RunStep(t, "verify the new config was set", func(t *testing.T) {
+				args := &structs.DCSpecificRequest{
+					Datacenter: "dc1",
+				}
+				var reply structs.CAConfiguration
+				require.NoError(t, msgpackrpc.CallWithCodec(codec, "ConnectCA.ConfigurationGet", args, &reply))
+
+				actual, err := ca.ParseConsulCAConfig(reply.Config)
+				require.NoError(t, err)
+				expected, err := ca.ParseConsulCAConfig(newConfig.Config)
+				require.NoError(t, err)
+				assert.Equal(t, reply.Provider, newConfig.Provider)
+				assert.Equal(t, actual, expected)
+			})
+
+			testutil.RunStep(t, "verify that new leaf certs get the cross-signed intermediate bundled", func(t *testing.T) {
+				// Generate a CSR and request signing
+				spiffeId := connect.TestSpiffeIDService(t, "web")
+				csr, _ := connect.TestCSR(t, spiffeId)
+				args := &structs.CASignRequest{
+					Datacenter: "dc1",
+					CSR:        csr,
+				}
+				var reply structs.IssuedCert
+				require.NoError(t, msgpackrpc.CallWithCodec(codec, "ConnectCA.Sign", args, &reply))
+
+				testutil.RunStep(t, "verify that the cert is signed by the new CA", func(t *testing.T) {
+					roots := x509.NewCertPool()
+					require.True(t, roots.AppendCertsFromPEM([]byte(newRootPEM)))
+					leaf, err := connect.ParseCert(reply.CertPEM)
+					require.NoError(t, err)
+					_, err = leaf.Verify(x509.VerifyOptions{
+						Roots: roots,
+					})
+					require.NoError(t, err)
+				})
+
+				testutil.RunStep(t, "and that it validates via the intermediate", func(t *testing.T) {
+					roots := x509.NewCertPool()
+					assert.True(t, roots.AppendCertsFromPEM([]byte(oldRoot.RootCert)))
+					leaf, err := connect.ParseCert(reply.CertPEM)
+					require.NoError(t, err)
+
+					// Make sure the intermediate was returned as well as leaf
+					_, rest := pem.Decode([]byte(reply.CertPEM))
+					require.NotEmpty(t, rest)
+
+					intermediates := x509.NewCertPool()
+					require.True(t, intermediates.AppendCertsFromPEM(rest))
+
+					_, err = leaf.Verify(x509.VerifyOptions{
+						Roots:         roots,
+						Intermediates: intermediates,
+					})
+					require.NoError(t, err)
+				})
+
+				testutil.RunStep(t, "verify other fields", func(t *testing.T) {
+					assert.Equal(t, "web", reply.Service)
+					assert.Equal(t, spiffeId.URI().String(), reply.ServiceURI)
+				})
+			})
+		})
+	}
+}
+
+func TestConnectCAConfig_Vault_TriggerRotation_Fails(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
+	ca.SkipIfVaultNotPresent(t)
+
+	t.Parallel()
+
+	testVault := ca.NewTestVaultServer(t)
+
+	token1 := ca.CreateVaultTokenWithAttrs(t, testVault.Client(), &ca.VaultTokenAttributes{
+		RootPath:         "pki-root",
+		IntermediatePath: "pki-primary",
+		ConsulManaged:    true,
+		WithSudo:         true,
+	})
+
+	token2 := ca.CreateVaultTokenWithAttrs(t, testVault.Client(), &ca.VaultTokenAttributes{
+		RootPath:         "pki-root",
+		IntermediatePath: "pki-intermediate",
+		ConsulManaged:    true,
+	})
+
+	newConfig := func(token string, keyType string, keyBits int) map[string]any {
+		return map[string]any{
+			"Address":             testVault.Addr,
+			"Token":               token,
+			"RootPKIPath":         "pki-root/",
+			"IntermediatePKIPath": "pki-intermediate/",
+			"PrivateKeyType":      keyType,
+			"PrivateKeyBits":      keyBits,
 		}
 	}
 
-	// Verify the new config was set.
-	{
-		args := &structs.DCSpecificRequest{
-			Datacenter: "dc1",
+	_, s1 := testServerWithConfig(t, func(c *Config) {
+		c.CAConfig = &structs.CAConfiguration{
+			Provider: "vault",
+			Config:   newConfig(token1, connect.DefaultPrivateKeyType, connect.DefaultPrivateKeyBits),
 		}
-		var reply structs.CAConfiguration
-		require.NoError(msgpackrpc.CallWithCodec(codec, "ConnectCA.ConfigurationGet", args, &reply))
+	})
+	testrpc.WaitForTestAgent(t, s1.RPC, "dc1")
 
-		actual, err := ca.ParseConsulCAConfig(reply.Config)
-		require.NoError(err)
-		expected, err := ca.ParseConsulCAConfig(newConfig.Config)
-		require.NoError(err)
-		assert.Equal(reply.Provider, newConfig.Provider)
-		assert.Equal(actual, expected)
+	// note: unlike many table tests, the ordering of these cases does matter
+	// because any non-errored case will modify the CA config, and any subsequent
+	// tests will use the same agent with that new CA config.
+	testSteps := []struct {
+		name      string
+		configFn  func() *structs.CAConfiguration
+		expectErr string
+	}{
+		{
+			name: "allow modifying key type and bits from default",
+			configFn: func() *structs.CAConfiguration {
+				return &structs.CAConfiguration{
+					Provider:                 "vault",
+					Config:                   newConfig(token2, "rsa", 4096),
+					ForceWithoutCrossSigning: true,
+				}
+			},
+		},
+		{
+			name: "error when trying to modify key bits",
+			configFn: func() *structs.CAConfiguration {
+				return &structs.CAConfiguration{
+					Provider:                 "vault",
+					Config:                   newConfig(token2, "rsa", 2048),
+					ForceWithoutCrossSigning: true,
+				}
+			},
+			expectErr: `cannot update the PrivateKeyBits field without changing RootPKIPath`,
+		},
+		{
+			name: "error when trying to modify key type",
+			configFn: func() *structs.CAConfiguration {
+				return &structs.CAConfiguration{
+					Provider:                 "vault",
+					Config:                   newConfig(token2, "ec", 256),
+					ForceWithoutCrossSigning: true,
+				}
+			},
+			expectErr: `cannot update the PrivateKeyType field without changing RootPKIPath`,
+		},
+		{
+			name: "allow update that does not change key type or bits",
+			configFn: func() *structs.CAConfiguration {
+				return &structs.CAConfiguration{
+					Provider:                 "vault",
+					Config:                   newConfig(token2, "rsa", 4096),
+					ForceWithoutCrossSigning: true,
+				}
+			},
+		},
 	}
 
-	// Verify that new leaf certs get the cross-signed intermediate bundled
-	{
-		// Generate a CSR and request signing
-		spiffeId := connect.TestSpiffeIDService(t, "web")
-		csr, _ := connect.TestCSR(t, spiffeId)
-		args := &structs.CASignRequest{
-			Datacenter: "dc1",
-			CSR:        csr,
-		}
-		var reply structs.IssuedCert
-		require.NoError(msgpackrpc.CallWithCodec(codec, "ConnectCA.Sign", args, &reply))
+	for _, tc := range testSteps {
+		testutil.RunStep(t, tc.name, func(t *testing.T) {
+			args := &structs.CARequest{
+				Datacenter: "dc1",
+				Config:     tc.configFn(),
+			}
+			var reply interface{}
 
-		// Verify that the cert is signed by the new CA
-		{
-			roots := x509.NewCertPool()
-			require.True(roots.AppendCertsFromPEM([]byte(newRootPEM)))
-			leaf, err := connect.ParseCert(reply.CertPEM)
-			require.NoError(err)
-			_, err = leaf.Verify(x509.VerifyOptions{
-				Roots: roots,
-			})
-			require.NoError(err)
-		}
-
-		// And that it validates via the intermediate
-		{
-			roots := x509.NewCertPool()
-			assert.True(roots.AppendCertsFromPEM([]byte(oldRoot.RootCert)))
-			leaf, err := connect.ParseCert(reply.CertPEM)
-			require.NoError(err)
-
-			// Make sure the intermediate was returned as well as leaf
-			_, rest := pem.Decode([]byte(reply.CertPEM))
-			require.NotEmpty(rest)
-
-			intermediates := x509.NewCertPool()
-			require.True(intermediates.AppendCertsFromPEM(rest))
-
-			_, err = leaf.Verify(x509.VerifyOptions{
-				Roots:         roots,
-				Intermediates: intermediates,
-			})
-			require.NoError(err)
-		}
-
-		// Verify other fields
-		assert.Equal("web", reply.Service)
-		assert.Equal(spiffeId.URI().String(), reply.ServiceURI)
+			codec := rpcClient(t, s1)
+			err := msgpackrpc.CallWithCodec(codec, "ConnectCA.ConfigurationSet", args, &reply)
+			if tc.expectErr == "" {
+				require.NoError(t, err)
+			} else {
+				testutil.RequireErrorContains(t, err, tc.expectErr)
+			}
+		})
 	}
 }
 
@@ -518,12 +672,10 @@ func TestConnectCAConfig_UpdateSecondary(t *testing.T) {
 
 	t.Parallel()
 
-	assert := assert.New(t)
-	require := require.New(t)
-
 	// Initialize primary as the primary DC
 	dir1, s1 := testServerWithConfig(t, func(c *Config) {
 		c.Datacenter = "primary"
+		c.PrimaryDatacenter = "primary"
 	})
 	defer os.RemoveAll(dir1)
 	defer s1.Shutdown()
@@ -546,24 +698,24 @@ func TestConnectCAConfig_UpdateSecondary(t *testing.T) {
 
 	// Capture the current root
 	rootList, activeRoot, err := getTestRoots(s1, "primary")
-	require.NoError(err)
-	require.Len(rootList.Roots, 1)
+	require.NoError(t, err)
+	require.Len(t, rootList.Roots, 1)
 	rootCert := activeRoot
 
-	waitForActiveCARoot(t, s1, rootCert)
-	waitForActiveCARoot(t, s2, rootCert)
+	testrpc.WaitForActiveCARoot(t, s1.RPC, "primary", rootCert)
+	testrpc.WaitForActiveCARoot(t, s2.RPC, "secondary", rootCert)
 
 	// Capture the current intermediate
 	rootList, activeRoot, err = getTestRoots(s2, "secondary")
-	require.NoError(err)
-	require.Len(rootList.Roots, 1)
-	require.Len(activeRoot.IntermediateCerts, 1)
+	require.NoError(t, err)
+	require.Len(t, rootList.Roots, 1)
+	require.Len(t, activeRoot.IntermediateCerts, 1)
 	oldIntermediatePEM := activeRoot.IntermediateCerts[0]
 
 	// Update the secondary CA config to use a new private key, which should
 	// cause a re-signing with a new intermediate.
 	_, newKey, err := connect.GeneratePrivateKey()
-	assert.NoError(err)
+	assert.NoError(t, err)
 	newConfig := &structs.CAConfiguration{
 		Provider: "consul",
 		Config: map[string]interface{}{
@@ -578,7 +730,7 @@ func TestConnectCAConfig_UpdateSecondary(t *testing.T) {
 		}
 		var reply interface{}
 
-		require.NoError(msgpackrpc.CallWithCodec(codec, "ConnectCA.ConfigurationSet", args, &reply))
+		require.NoError(t, msgpackrpc.CallWithCodec(codec, "ConnectCA.ConfigurationSet", args, &reply))
 	}
 
 	// Make sure the new intermediate has replaced the old one in the active root,
@@ -589,12 +741,12 @@ func TestConnectCAConfig_UpdateSecondary(t *testing.T) {
 			Datacenter: "secondary",
 		}
 		var reply structs.IndexedCARoots
-		require.Nil(msgpackrpc.CallWithCodec(codec, "ConnectCA.Roots", args, &reply))
-		require.Len(reply.Roots, 1)
-		require.Len(reply.Roots[0].IntermediateCerts, 1)
+		require.Nil(t, msgpackrpc.CallWithCodec(codec, "ConnectCA.Roots", args, &reply))
+		require.Len(t, reply.Roots, 1)
+		require.Len(t, reply.Roots[0].IntermediateCerts, 1)
 		newIntermediatePEM = reply.Roots[0].IntermediateCerts[0]
-		require.NotEqual(oldIntermediatePEM, newIntermediatePEM)
-		require.Equal(reply.Roots[0].RootCert, rootCert.RootCert)
+		require.NotEqual(t, oldIntermediatePEM, newIntermediatePEM)
+		require.Equal(t, reply.Roots[0].RootCert, rootCert.RootCert)
 	}
 
 	// Verify the new config was set.
@@ -603,14 +755,14 @@ func TestConnectCAConfig_UpdateSecondary(t *testing.T) {
 			Datacenter: "secondary",
 		}
 		var reply structs.CAConfiguration
-		require.NoError(msgpackrpc.CallWithCodec(codec, "ConnectCA.ConfigurationGet", args, &reply))
+		require.NoError(t, msgpackrpc.CallWithCodec(codec, "ConnectCA.ConfigurationGet", args, &reply))
 
 		actual, err := ca.ParseConsulCAConfig(reply.Config)
-		require.NoError(err)
+		require.NoError(t, err)
 		expected, err := ca.ParseConsulCAConfig(newConfig.Config)
-		require.NoError(err)
-		assert.Equal(reply.Provider, newConfig.Provider)
-		assert.Equal(actual, expected)
+		require.NoError(t, err)
+		assert.Equal(t, reply.Provider, newConfig.Provider)
+		assert.Equal(t, actual, expected)
 	}
 
 	// Verify that new leaf certs get the new intermediate bundled
@@ -623,28 +775,28 @@ func TestConnectCAConfig_UpdateSecondary(t *testing.T) {
 			CSR:        csr,
 		}
 		var reply structs.IssuedCert
-		require.NoError(msgpackrpc.CallWithCodec(codec, "ConnectCA.Sign", args, &reply))
+		require.NoError(t, msgpackrpc.CallWithCodec(codec, "ConnectCA.Sign", args, &reply))
 
 		// Verify the leaf cert has the new intermediate.
 		{
 			roots := x509.NewCertPool()
-			assert.True(roots.AppendCertsFromPEM([]byte(rootCert.RootCert)))
+			assert.True(t, roots.AppendCertsFromPEM([]byte(rootCert.RootCert)))
 			leaf, err := connect.ParseCert(reply.CertPEM)
-			require.NoError(err)
+			require.NoError(t, err)
 
 			intermediates := x509.NewCertPool()
-			require.True(intermediates.AppendCertsFromPEM([]byte(newIntermediatePEM)))
+			require.True(t, intermediates.AppendCertsFromPEM([]byte(newIntermediatePEM)))
 
 			_, err = leaf.Verify(x509.VerifyOptions{
 				Roots:         roots,
 				Intermediates: intermediates,
 			})
-			require.NoError(err)
+			require.NoError(t, err)
 		}
 
 		// Verify other fields
-		assert.Equal("web", reply.Service)
-		assert.Equal(spiffeId.URI().String(), reply.ServiceURI)
+		assert.Equal(t, "web", reply.Service)
+		assert.Equal(t, spiffeId.URI().String(), reply.ServiceURI)
 	}
 
 	// Update a minor field in the config that doesn't trigger an intermediate refresh.
@@ -663,7 +815,7 @@ func TestConnectCAConfig_UpdateSecondary(t *testing.T) {
 			}
 			var reply interface{}
 
-			require.NoError(msgpackrpc.CallWithCodec(codec, "ConnectCA.ConfigurationSet", args, &reply))
+			require.NoError(t, msgpackrpc.CallWithCodec(codec, "ConnectCA.ConfigurationSet", args, &reply))
 		}
 	}
 }
@@ -693,9 +845,8 @@ func TestConnectCASign(t *testing.T) {
 
 	for _, tt := range tests {
 		t.Run(fmt.Sprintf("%s-%d", tt.caKeyType, tt.caKeyBits), func(t *testing.T) {
-			assert := assert.New(t)
-			require := require.New(t)
 			dir1, s1 := testServerWithConfig(t, func(cfg *Config) {
+				cfg.PrimaryDatacenter = "dc1"
 				cfg.CAConfig.Config["PrivateKeyType"] = tt.caKeyType
 				cfg.CAConfig.Config["PrivateKeyBits"] = tt.caKeyBits
 			})
@@ -716,7 +867,7 @@ func TestConnectCASign(t *testing.T) {
 				CSR:        csr,
 			}
 			var reply structs.IssuedCert
-			require.NoError(msgpackrpc.CallWithCodec(codec, "ConnectCA.Sign", args, &reply))
+			require.NoError(t, msgpackrpc.CallWithCodec(codec, "ConnectCA.Sign", args, &reply))
 
 			// Generate a second CSR and request signing
 			spiffeId2 := connect.TestSpiffeIDService(t, "web2")
@@ -727,20 +878,20 @@ func TestConnectCASign(t *testing.T) {
 			}
 
 			var reply2 structs.IssuedCert
-			require.NoError(msgpackrpc.CallWithCodec(codec, "ConnectCA.Sign", args, &reply2))
-			require.True(reply2.ModifyIndex > reply.ModifyIndex)
+			require.NoError(t, msgpackrpc.CallWithCodec(codec, "ConnectCA.Sign", args, &reply2))
+			require.True(t, reply2.ModifyIndex > reply.ModifyIndex)
 
 			// Get the current CA
 			state := s1.fsm.State()
 			_, ca, err := state.CARootActive(nil)
-			require.NoError(err)
+			require.NoError(t, err)
 
 			// Verify that the cert is signed by the CA
-			require.NoError(connect.ValidateLeaf(ca.RootCert, reply.CertPEM, nil))
+			require.NoError(t, connect.ValidateLeaf(ca.RootCert, reply.CertPEM, nil))
 
 			// Verify other fields
-			assert.Equal("web", reply.Service)
-			assert.Equal(spiffeId.URI().String(), reply.ServiceURI)
+			assert.Equal(t, "web", reply.Service)
+			assert.Equal(t, spiffeId.URI().String(), reply.ServiceURI)
 		})
 	}
 }
@@ -751,7 +902,6 @@ func TestConnectCASign(t *testing.T) {
 func BenchmarkConnectCASign(b *testing.B) {
 	t := &testing.T{}
 
-	require := require.New(b)
 	dir1, s1 := testServer(t)
 	defer os.RemoveAll(dir1)
 	defer s1.Shutdown()
@@ -771,7 +921,9 @@ func BenchmarkConnectCASign(b *testing.B) {
 
 	b.ResetTimer()
 	for n := 0; n < b.N; n++ {
-		require.NoError(msgpackrpc.CallWithCodec(codec, "ConnectCA.Sign", args, &reply))
+		if err := msgpackrpc.CallWithCodec(codec, "ConnectCA.Sign", args, &reply); err != nil {
+			b.Fatalf("err: %v", err)
+		}
 	}
 }
 
@@ -782,9 +934,9 @@ func TestConnectCASign_rateLimit(t *testing.T) {
 
 	t.Parallel()
 
-	require := require.New(t)
 	dir1, s1 := testServerWithConfig(t, func(c *Config) {
 		c.Datacenter = "dc1"
+		c.PrimaryDatacenter = "dc1"
 		c.Bootstrap = true
 		c.CAConfig.Config = map[string]interface{}{
 			// It actually doesn't work as expected with some higher values because
@@ -826,7 +978,7 @@ func TestConnectCASign_rateLimit(t *testing.T) {
 		} else if err.Error() == ErrRateLimited.Error() {
 			limitedCount++
 		} else {
-			require.NoError(err)
+			require.NoError(t, err)
 		}
 	}
 	// I've only ever seen this as 1/9 however if the test runs slowly on an
@@ -836,8 +988,8 @@ func TestConnectCASign_rateLimit(t *testing.T) {
 	// check that some limiting is being applied. Note that we can't just measure
 	// the time it took to send them all and infer how many should have succeeded
 	// without some complex modeling of the token bucket algorithm.
-	require.Truef(successCount >= 1, "at least 1 CSRs should have succeeded, got %d", successCount)
-	require.Truef(limitedCount >= 7, "at least 7 CSRs should have been rate limited, got %d", limitedCount)
+	require.Truef(t, successCount >= 1, "at least 1 CSRs should have succeeded, got %d", successCount)
+	require.Truef(t, limitedCount >= 7, "at least 7 CSRs should have been rate limited, got %d", limitedCount)
 }
 
 func TestConnectCASign_concurrencyLimit(t *testing.T) {
@@ -847,9 +999,9 @@ func TestConnectCASign_concurrencyLimit(t *testing.T) {
 
 	t.Parallel()
 
-	require := require.New(t)
 	dir1, s1 := testServerWithConfig(t, func(c *Config) {
 		c.Datacenter = "dc1"
+		c.PrimaryDatacenter = "dc1"
 		c.Bootstrap = true
 		c.CAConfig.Config = map[string]interface{}{
 			// Must disable the rate limit since it takes precedence
@@ -906,7 +1058,7 @@ func TestConnectCASign_concurrencyLimit(t *testing.T) {
 		} else if err.Error() == ErrRateLimited.Error() {
 			limitedCount++
 		} else {
-			require.NoError(err)
+			require.NoError(t, err)
 		}
 	}
 
@@ -945,7 +1097,7 @@ func TestConnectCASign_concurrencyLimit(t *testing.T) {
 	// requests were serialized.
 	t.Logf("min=%s, max=%s", minTime, maxTime)
 	//t.Fail() // Uncomment to see the time spread logged
-	require.Truef(successCount >= 1, "at least 1 CSRs should have succeeded, got %d", successCount)
+	require.Truef(t, successCount >= 1, "at least 1 CSRs should have succeeded, got %d", successCount)
 }
 
 func TestConnectCASignValidation(t *testing.T) {
@@ -956,10 +1108,10 @@ func TestConnectCASignValidation(t *testing.T) {
 	t.Parallel()
 
 	dir1, s1 := testServerWithConfig(t, func(c *Config) {
-		c.ACLDatacenter = "dc1"
+		c.PrimaryDatacenter = "dc1"
 		c.ACLsEnabled = true
-		c.ACLMasterToken = "root"
-		c.ACLDefaultPolicy = "deny"
+		c.ACLInitialManagementToken = "root"
+		c.ACLResolverSettings.ACLDefaultPolicy = "deny"
 	})
 	defer os.RemoveAll(dir1)
 	defer s1.Shutdown()
@@ -968,25 +1120,7 @@ func TestConnectCASignValidation(t *testing.T) {
 
 	testrpc.WaitForLeader(t, s1.RPC, "dc1")
 
-	// Create an ACL token with service:write for web*
-	var webToken string
-	{
-		arg := structs.ACLRequest{
-			Datacenter: "dc1",
-			Op:         structs.ACLSet,
-			ACL: structs.ACL{
-				Name: "User token",
-				Type: structs.ACLTokenTypeClient,
-				Rules: `
-				service "web" {
-					policy = "write"
-				}`,
-			},
-			WriteRequest: structs.WriteRequest{Token: "root"},
-		}
-		require.NoError(t, msgpackrpc.CallWithCodec(codec, "ACL.Apply", &arg, &webToken))
-	}
-
+	webToken := createToken(t, codec, `service "web" { policy = "write" }`)
 	testWebID := connect.TestSpiffeIDService(t, "web")
 
 	tests := []struct {
