@@ -19,13 +19,14 @@ import (
 	"testing"
 	"time"
 
-	msgpackrpc "github.com/hashicorp/consul-net-rpc/net-rpc-msgpackrpc"
-	"github.com/hashicorp/consul-net-rpc/net/rpc"
-	vaultapi "github.com/hashicorp/vault/api"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 	"google.golang.org/grpc"
 
+	msgpackrpc "github.com/hashicorp/consul-net-rpc/net-rpc-msgpackrpc"
+	"github.com/hashicorp/consul-net-rpc/net/rpc"
+	vaultapi "github.com/hashicorp/vault/api"
+
 	"github.com/hashicorp/consul/acl"
 	"github.com/hashicorp/consul/agent/connect"
 	"github.com/hashicorp/consul/agent/connect/ca"
@@ -566,7 +567,7 @@ func TestCAManager_Initialize_Logging(t *testing.T) {
 	deps := newDefaultDeps(t, conf1)
 	deps.Logger = logger
 
-	s1, err := NewServer(conf1, deps, grpc.NewServer(), nil, logger, nil)
+	s1, err := NewServer(conf1, deps, grpc.NewServer(), nil, logger)
 	require.NoError(t, err)
 	defer s1.Shutdown()
 	testrpc.WaitForLeader(t, s1.RPC, "dc1")
@@ -1317,12 +1318,6 @@ func TestCAManager_AuthorizeAndSignCertificate(t *testing.T) {
 		Host:       "test-host",
 		Partition:  "test-partition",
 	}.URI()
-	identityURL := connect.SpiffeIDWorkloadIdentity{
-		TrustDomain:      "test-trust-domain",
-		Partition:        "test-partition",
-		Namespace:        "test-namespace",
-		WorkloadIdentity: "test-workload-identity",
-	}.URI()
 
 	tests := []struct {
 		name      string
@@ -1418,15 +1413,6 @@ func TestCAManager_AuthorizeAndSignCertificate(t *testing.T) {
 				}
 			},
 		},
-		{
-			name:      "err_identity_write_not_allowed",
-			expectErr: "Permission denied",
-			getCSR: func() *x509.CertificateRequest {
-				return &x509.CertificateRequest{
-					URIs: []*url.URL{identityURL},
-				}
-			},
-		},
 	}
 
 	for _, tc := range tests {
