@@ -1,3 +1,6 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: BUSL-1.1
+
 package consul
 
 import (
@@ -5,18 +8,26 @@ import (
 	"testing"
 	"time"
 
+	"github.com/stretchr/testify/require"
+
+	msgpackrpc "github.com/hashicorp/consul-net-rpc/net-rpc-msgpackrpc"
+
 	"github.com/hashicorp/consul/acl"
 	"github.com/hashicorp/consul/agent/structs"
-	"github.com/hashicorp/consul/lib"
+	"github.com/hashicorp/consul/lib/stringslice"
 	"github.com/hashicorp/consul/testrpc"
-	"github.com/hashicorp/net-rpc-msgpackrpc"
 )
 
 func TestSession_Apply(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	dir1, s1 := testServer(t)
 	defer os.RemoveAll(dir1)
 	defer s1.Shutdown()
+
 	codec := rpcClient(t, s1)
 	defer codec.Close()
 
@@ -41,7 +52,7 @@ func TestSession_Apply(t *testing.T) {
 
 	// Verify
 	state := s1.fsm.State()
-	_, s, err := state.SessionGet(nil, out)
+	_, s, err := state.SessionGet(nil, out, nil)
 	if err != nil {
 		t.Fatalf("err: %v", err)
 	}
@@ -63,7 +74,7 @@ func TestSession_Apply(t *testing.T) {
 	}
 
 	// Verify
-	_, s, err = state.SessionGet(nil, id)
+	_, s, err = state.SessionGet(nil, id, nil)
 	if err != nil {
 		t.Fatalf("err: %v", err)
 	}
@@ -73,10 +84,15 @@ func TestSession_Apply(t *testing.T) {
 }
 
 func TestSession_DeleteApply(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	dir1, s1 := testServer(t)
 	defer os.RemoveAll(dir1)
 	defer s1.Shutdown()
+
 	codec := rpcClient(t, s1)
 	defer codec.Close()
 
@@ -102,7 +118,7 @@ func TestSession_DeleteApply(t *testing.T) {
 
 	// Verify
 	state := s1.fsm.State()
-	_, s, err := state.SessionGet(nil, out)
+	_, s, err := state.SessionGet(nil, out, nil)
 	if err != nil {
 		t.Fatalf("err: %v", err)
 	}
@@ -127,7 +143,7 @@ func TestSession_DeleteApply(t *testing.T) {
 	}
 
 	// Verify
-	_, s, err = state.SessionGet(nil, id)
+	_, s, err = state.SessionGet(nil, id, nil)
 	if err != nil {
 		t.Fatalf("err: %v", err)
 	}
@@ -137,46 +153,36 @@ func TestSession_DeleteApply(t *testing.T) {
 }
 
 func TestSession_Apply_ACLDeny(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	dir1, s1 := testServerWithConfig(t, func(c *Config) {
-		c.ACLDatacenter = "dc1"
-		c.ACLMasterToken = "root"
-		c.ACLDefaultPolicy = "deny"
-		c.ACLEnforceVersion8 = false
+		c.PrimaryDatacenter = "dc1"
+		c.ACLsEnabled = true
+		c.ACLInitialManagementToken = "root"
+		c.ACLResolverSettings.ACLDefaultPolicy = "deny"
 	})
 	defer os.RemoveAll(dir1)
 	defer s1.Shutdown()
+
 	codec := rpcClient(t, s1)
 	defer codec.Close()
 
-	testrpc.WaitForLeader(t, s1.RPC, "dc1")
+	testrpc.WaitForLeader(t, s1.RPC, "dc1", testrpc.WithToken("root"))
 
-	// Create the ACL.
-	req := structs.ACLRequest{
-		Datacenter: "dc1",
-		Op:         structs.ACLSet,
-		ACL: structs.ACL{
-			Name: "User token",
-			Type: structs.ACLTypeClient,
-			Rules: `
+	rules := `
 session "foo" {
 	policy = "write"
 }
-`,
-		},
-		WriteRequest: structs.WriteRequest{Token: "root"},
-	}
-
-	var token string
-	if err := msgpackrpc.CallWithCodec(codec, "ACL.Apply", &req, &token); err != nil {
-		t.Fatalf("err: %v", err)
-	}
+`
+	token := createToken(t, codec, rules)
 
 	// Just add a node.
 	s1.fsm.State().EnsureNode(1, &structs.Node{Node: "foo", Address: "127.0.0.1"})
 
-	// Try to create without a token, which will go through since version 8
-	// enforcement isn't enabled.
+	// Try to create without a token, which will be denied.
 	arg := structs.SessionRequest{
 		Datacenter: "dc1",
 		Op:         structs.SessionCreate,
@@ -185,40 +191,23 @@ session "foo" {
 			Name: "my-session",
 		},
 	}
-	var id1 string
-	if err := msgpackrpc.CallWithCodec(codec, "Session.Apply", &arg, &id1); err != nil {
-		t.Fatalf("err: %v", err)
-	}
-
-	// Now turn on version 8 enforcement and try again, it should be denied.
-	var id2 string
-	s1.config.ACLEnforceVersion8 = true
-	err := msgpackrpc.CallWithCodec(codec, "Session.Apply", &arg, &id2)
+	var id string
+	err := msgpackrpc.CallWithCodec(codec, "Session.Apply", &arg, &id)
 	if !acl.IsErrPermissionDenied(err) {
 		t.Fatalf("err: %v", err)
 	}
 
 	// Now set a token and try again. This should go through.
 	arg.Token = token
-	if err := msgpackrpc.CallWithCodec(codec, "Session.Apply", &arg, &id2); err != nil {
+	if err := msgpackrpc.CallWithCodec(codec, "Session.Apply", &arg, &id); err != nil {
 		t.Fatalf("err: %v", err)
 	}
 
-	// Do a delete on the first session with version 8 enforcement off and
-	// no token. This should go through.
+	// Make sure the delete of the session fails without a token.
 	var out string
-	s1.config.ACLEnforceVersion8 = false
 	arg.Op = structs.SessionDestroy
+	arg.Session.ID = id
 	arg.Token = ""
-	arg.Session.ID = id1
-	if err := msgpackrpc.CallWithCodec(codec, "Session.Apply", &arg, &out); err != nil {
-		t.Fatalf("err: %v", err)
-	}
-
-	// Turn on version 8 enforcement and make sure the delete of the second
-	// session fails.
-	s1.config.ACLEnforceVersion8 = true
-	arg.Session.ID = id2
 	err = msgpackrpc.CallWithCodec(codec, "Session.Apply", &arg, &out)
 	if !acl.IsErrPermissionDenied(err) {
 		t.Fatalf("err: %v", err)
@@ -232,10 +221,15 @@ session "foo" {
 }
 
 func TestSession_Get(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	dir1, s1 := testServer(t)
 	defer os.RemoveAll(dir1)
 	defer s1.Shutdown()
+
 	codec := rpcClient(t, s1)
 	defer codec.Close()
 
@@ -256,7 +250,57 @@ func TestSession_Get(t *testing.T) {
 
 	getR := structs.SessionSpecificRequest{
 		Datacenter: "dc1",
-		Session:    out,
+		SessionID:  out,
+	}
+	var sessions structs.IndexedSessions
+	if err := msgpackrpc.CallWithCodec(codec, "Session.Get", &getR, &sessions); err != nil {
+		t.Fatalf("err: %v", err)
+	}
+
+	if sessions.Index == 0 {
+		t.Fatalf("Bad: %v", sessions)
+	}
+	if len(sessions.Sessions) != 1 {
+		t.Fatalf("Bad: %v", sessions)
+	}
+	s := sessions.Sessions[0]
+	if s.ID != out {
+		t.Fatalf("bad: %v", s)
+	}
+}
+
+func TestSession_Get_Compat(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
+	t.Parallel()
+	dir1, s1 := testServer(t)
+	defer os.RemoveAll(dir1)
+	defer s1.Shutdown()
+
+	codec := rpcClient(t, s1)
+	defer codec.Close()
+
+	testrpc.WaitForLeader(t, s1.RPC, "dc1")
+
+	s1.fsm.State().EnsureNode(1, &structs.Node{Node: "foo", Address: "127.0.0.1"})
+	arg := structs.SessionRequest{
+		Datacenter: "dc1",
+		Op:         structs.SessionCreate,
+		Session: structs.Session{
+			Node: "foo",
+		},
+	}
+	var out string
+	if err := msgpackrpc.CallWithCodec(codec, "Session.Apply", &arg, &out); err != nil {
+		t.Fatalf("err: %v", err)
+	}
+
+	getR := structs.SessionSpecificRequest{
+		Datacenter: "dc1",
+		// this should get converted to the SessionID field internally
+		Session: out,
 	}
 	var sessions structs.IndexedSessions
 	if err := msgpackrpc.CallWithCodec(codec, "Session.Get", &getR, &sessions); err != nil {
@@ -276,10 +320,15 @@ func TestSession_Get(t *testing.T) {
 }
 
 func TestSession_List(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	dir1, s1 := testServer(t)
 	defer os.RemoveAll(dir1)
 	defer s1.Shutdown()
+
 	codec := rpcClient(t, s1)
 	defer codec.Close()
 
@@ -318,7 +367,7 @@ func TestSession_List(t *testing.T) {
 	}
 	for i := 0; i < len(sessions.Sessions); i++ {
 		s := sessions.Sessions[i]
-		if !lib.StrContains(ids, s.ID) {
+		if !stringslice.Contains(ids, s.ID) {
 			t.Fatalf("bad: %v", s)
 		}
 		if s.Node != "foo" {
@@ -328,40 +377,37 @@ func TestSession_List(t *testing.T) {
 }
 
 func TestSession_Get_List_NodeSessions_ACLFilter(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
+
 	dir1, s1 := testServerWithConfig(t, func(c *Config) {
-		c.ACLDatacenter = "dc1"
-		c.ACLMasterToken = "root"
-		c.ACLDefaultPolicy = "deny"
-		c.ACLEnforceVersion8 = false
+		c.PrimaryDatacenter = "dc1"
+		c.ACLsEnabled = true
+		c.ACLInitialManagementToken = "root"
+		c.ACLResolverSettings.ACLDefaultPolicy = "deny"
 	})
 	defer os.RemoveAll(dir1)
 	defer s1.Shutdown()
+
 	codec := rpcClient(t, s1)
 	defer codec.Close()
 
-	testrpc.WaitForLeader(t, s1.RPC, "dc1")
+	testrpc.WaitForLeader(t, s1.RPC, "dc1", testrpc.WithToken("root"))
 
-	// Create the ACL.
-	req := structs.ACLRequest{
-		Datacenter: "dc1",
-		Op:         structs.ACLSet,
-		ACL: structs.ACL{
-			Name: "User token",
-			Type: structs.ACLTypeClient,
-			Rules: `
-session "foo" {
-	policy = "read"
-}
-`,
-		},
-		WriteRequest: structs.WriteRequest{Token: "root"},
-	}
+	deniedToken := createTokenWithPolicyName(t, codec, "denied", `
+		session "foo" {
+			policy = "deny"
+		}
+	`, "root")
 
-	var token string
-	if err := msgpackrpc.CallWithCodec(codec, "ACL.Apply", &req, &token); err != nil {
-		t.Fatalf("err: %v", err)
-	}
+	allowedToken := createTokenWithPolicyName(t, codec, "allowed", `
+		session "foo" {
+			policy = "read"
+		}
+	`, "root")
 
 	// Create a node and a session.
 	s1.fsm.State().EnsureNode(1, &structs.Node{Node: "foo", Address: "127.0.0.1"})
@@ -374,137 +420,108 @@ session "foo" {
 		WriteRequest: structs.WriteRequest{Token: "root"},
 	}
 	var out string
-	if err := msgpackrpc.CallWithCodec(codec, "Session.Apply", &arg, &out); err != nil {
-		t.Fatalf("err: %v", err)
-	}
+	err := msgpackrpc.CallWithCodec(codec, "Session.Apply", &arg, &out)
+	require.NoError(t, err)
 
-	// Perform all the read operations, which should go through since version
-	// 8 ACL enforcement isn't enabled.
-	getR := structs.SessionSpecificRequest{
-		Datacenter: "dc1",
-		Session:    out,
-	}
-	{
-		var sessions structs.IndexedSessions
-		if err := msgpackrpc.CallWithCodec(codec, "Session.Get", &getR, &sessions); err != nil {
-			t.Fatalf("err: %v", err)
-		}
-		if len(sessions.Sessions) != 1 {
-			t.Fatalf("bad: %v", sessions.Sessions)
-		}
-	}
-	listR := structs.DCSpecificRequest{
-		Datacenter: "dc1",
-	}
-	{
-		var sessions structs.IndexedSessions
-		if err := msgpackrpc.CallWithCodec(codec, "Session.List", &listR, &sessions); err != nil {
-			t.Fatalf("err: %v", err)
-		}
-		if len(sessions.Sessions) != 1 {
-			t.Fatalf("bad: %v", sessions.Sessions)
-		}
-	}
-	nodeR := structs.NodeSpecificRequest{
-		Datacenter: "dc1",
-		Node:       "foo",
-	}
-	{
-		var sessions structs.IndexedSessions
-		if err := msgpackrpc.CallWithCodec(codec, "Session.NodeSessions", &nodeR, &sessions); err != nil {
-			t.Fatalf("err: %v", err)
-		}
-		if len(sessions.Sessions) != 1 {
-			t.Fatalf("bad: %v", sessions.Sessions)
-		}
-	}
+	t.Run("Get", func(t *testing.T) {
 
-	// Now turn on version 8 enforcement and make sure everything is empty.
-	s1.config.ACLEnforceVersion8 = true
-	{
-		var sessions structs.IndexedSessions
-		if err := msgpackrpc.CallWithCodec(codec, "Session.Get", &getR, &sessions); err != nil {
-			t.Fatalf("err: %v", err)
+		req := &structs.SessionSpecificRequest{
+			Datacenter: "dc1",
+			SessionID:  out,
 		}
-		if len(sessions.Sessions) != 0 {
-			t.Fatalf("bad: %v", sessions.Sessions)
-		}
-	}
-	{
+		req.Token = deniedToken
+
+		// ACL-restricted results filtered out.
 		var sessions structs.IndexedSessions
 
-		if err := msgpackrpc.CallWithCodec(codec, "Session.List", &listR, &sessions); err != nil {
-			t.Fatalf("err: %v", err)
-		}
-		if len(sessions.Sessions) != 0 {
-			t.Fatalf("bad: %v", sessions.Sessions)
-		}
-	}
-	{
-		var sessions structs.IndexedSessions
-		if err := msgpackrpc.CallWithCodec(codec, "Session.NodeSessions", &nodeR, &sessions); err != nil {
-			t.Fatalf("err: %v", err)
-		}
-		if len(sessions.Sessions) != 0 {
-			t.Fatalf("bad: %v", sessions.Sessions)
-		}
-	}
+		err := msgpackrpc.CallWithCodec(codec, "Session.Get", req, &sessions)
+		require.NoError(t, err)
+		require.Empty(t, sessions.Sessions)
+		require.True(t, sessions.QueryMeta.ResultsFilteredByACLs, "ResultsFilteredByACLs should be true")
 
-	// Finally, supply the token and make sure the reads are allowed.
-	getR.Token = token
-	{
-		var sessions structs.IndexedSessions
-		if err := msgpackrpc.CallWithCodec(codec, "Session.Get", &getR, &sessions); err != nil {
-			t.Fatalf("err: %v", err)
-		}
-		if len(sessions.Sessions) != 1 {
-			t.Fatalf("bad: %v", sessions.Sessions)
-		}
-	}
-	listR.Token = token
-	{
-		var sessions structs.IndexedSessions
-		if err := msgpackrpc.CallWithCodec(codec, "Session.List", &listR, &sessions); err != nil {
-			t.Fatalf("err: %v", err)
-		}
-		if len(sessions.Sessions) != 1 {
-			t.Fatalf("bad: %v", sessions.Sessions)
-		}
-	}
-	nodeR.Token = token
-	{
-		var sessions structs.IndexedSessions
-		if err := msgpackrpc.CallWithCodec(codec, "Session.NodeSessions", &nodeR, &sessions); err != nil {
-			t.Fatalf("err: %v", err)
-		}
-		if len(sessions.Sessions) != 1 {
-			t.Fatalf("bad: %v", sessions.Sessions)
-		}
-	}
+		// ACL-restricted results included.
+		req.Token = allowedToken
 
-	// Try to get a session that doesn't exist to make sure that's handled
-	// correctly by the filter (it will get passed a nil slice).
-	getR.Session = "adf4238a-882b-9ddc-4a9d-5b6758e4159e"
-	{
+		err = msgpackrpc.CallWithCodec(codec, "Session.Get", req, &sessions)
+		require.NoError(t, err)
+		require.Len(t, sessions.Sessions, 1)
+		require.False(t, sessions.QueryMeta.ResultsFilteredByACLs, "ResultsFilteredByACLs should be false")
+
+		// Try to get a session that doesn't exist to make sure that's handled
+		// correctly by the filter (it will get passed a nil slice).
+		req.SessionID = "adf4238a-882b-9ddc-4a9d-5b6758e4159e"
+
+		err = msgpackrpc.CallWithCodec(codec, "Session.Get", req, &sessions)
+		require.NoError(t, err)
+		require.Empty(t, sessions.Sessions)
+		require.False(t, sessions.QueryMeta.ResultsFilteredByACLs, "ResultsFilteredByACLs should be false")
+	})
+
+	t.Run("List", func(t *testing.T) {
+
+		req := &structs.DCSpecificRequest{
+			Datacenter: "dc1",
+		}
+		req.Token = deniedToken
+
+		// ACL-restricted results filtered out.
 		var sessions structs.IndexedSessions
-		if err := msgpackrpc.CallWithCodec(codec, "Session.Get", &getR, &sessions); err != nil {
-			t.Fatalf("err: %v", err)
+
+		err := msgpackrpc.CallWithCodec(codec, "Session.List", req, &sessions)
+		require.NoError(t, err)
+		require.Empty(t, sessions.Sessions)
+		require.True(t, sessions.QueryMeta.ResultsFilteredByACLs, "ResultsFilteredByACLs should be true")
+
+		// ACL-restricted results included.
+		req.Token = allowedToken
+
+		err = msgpackrpc.CallWithCodec(codec, "Session.List", req, &sessions)
+		require.NoError(t, err)
+		require.Len(t, sessions.Sessions, 1)
+		require.False(t, sessions.QueryMeta.ResultsFilteredByACLs, "ResultsFilteredByACLs should be false")
+	})
+
+	t.Run("NodeSessions", func(t *testing.T) {
+
+		req := &structs.NodeSpecificRequest{
+			Datacenter: "dc1",
+			Node:       "foo",
 		}
-		if len(sessions.Sessions) != 0 {
-			t.Fatalf("bad: %v", sessions.Sessions)
-		}
-	}
+		req.Token = deniedToken
+
+		// ACL-restricted results filtered out.
+		var sessions structs.IndexedSessions
+
+		err := msgpackrpc.CallWithCodec(codec, "Session.NodeSessions", req, &sessions)
+		require.NoError(t, err)
+		require.Empty(t, sessions.Sessions)
+		require.True(t, sessions.QueryMeta.ResultsFilteredByACLs, "ResultsFilteredByACLs should be true")
+
+		// ACL-restricted results included.
+		req.Token = allowedToken
+
+		err = msgpackrpc.CallWithCodec(codec, "Session.NodeSessions", req, &sessions)
+		require.NoError(t, err)
+		require.Len(t, sessions.Sessions, 1)
+		require.False(t, sessions.QueryMeta.ResultsFilteredByACLs, "ResultsFilteredByACLs should be false")
+	})
 }
 
 func TestSession_ApplyTimers(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	dir1, s1 := testServer(t)
 	defer os.RemoveAll(dir1)
 	defer s1.Shutdown()
-	testrpc.WaitForTestAgent(t, s1.RPC, "dc1")
+
 	codec := rpcClient(t, s1)
 	defer codec.Close()
 
+	testrpc.WaitForTestAgent(t, s1.RPC, "dc1")
+
 	s1.fsm.State().EnsureNode(1, &structs.Node{Node: "foo", Address: "127.0.0.1"})
 	arg := structs.SessionRequest{
 		Datacenter: "dc1",
@@ -538,6 +555,10 @@ func TestSession_ApplyTimers(t *testing.T) {
 }
 
 func TestSession_Renew(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	// This method is timing sensitive, disable Parallel
 	//t.Parallel()
 	ttl := 1 * time.Second
@@ -549,6 +570,7 @@ func TestSession_Renew(t *testing.T) {
 	defer os.RemoveAll(dir1)
 	defer s1.Shutdown()
 	testrpc.WaitForTestAgent(t, s1.RPC, "dc1")
+
 	codec := rpcClient(t, s1)
 	defer codec.Close()
 
@@ -592,7 +614,7 @@ func TestSession_Renew(t *testing.T) {
 	}
 	for i := 0; i < len(sessions.Sessions); i++ {
 		s := sessions.Sessions[i]
-		if !lib.StrContains(ids, s.ID) {
+		if !stringslice.Contains(ids, s.ID) {
 			t.Fatalf("bad: %v", s)
 		}
 		if s.Node != "foo" {
@@ -611,7 +633,7 @@ func TestSession_Renew(t *testing.T) {
 	for i := 0; i < 3; i++ {
 		renewR := structs.SessionSpecificRequest{
 			Datacenter: "dc1",
-			Session:    ids[i],
+			SessionID:  ids[i],
 		}
 		var session structs.IndexedSessions
 		if err := msgpackrpc.CallWithCodec(codec, "Session.Renew", &renewR, &session); err != nil {
@@ -626,7 +648,7 @@ func TestSession_Renew(t *testing.T) {
 		}
 
 		s := session.Sessions[0]
-		if !lib.StrContains(ids, s.ID) {
+		if !stringslice.Contains(ids, s.ID) {
 			t.Fatalf("bad: %v", s)
 		}
 		if s.Node != "foo" {
@@ -653,7 +675,7 @@ func TestSession_Renew(t *testing.T) {
 
 	for i := 0; i < len(sessionsL1.Sessions); i++ {
 		s := sessionsL1.Sessions[i]
-		if !lib.StrContains(ids, s.ID) {
+		if !stringslice.Contains(ids, s.ID) {
 			t.Fatalf("bad: %v", s)
 		}
 		if s.Node != "foo" {
@@ -685,7 +707,7 @@ func TestSession_Renew(t *testing.T) {
 	if len(sessionsL2.Sessions) != 0 {
 		for i := 0; i < len(sessionsL2.Sessions); i++ {
 			s := sessionsL2.Sessions[i]
-			if !lib.StrContains(ids, s.ID) {
+			if !stringslice.Contains(ids, s.ID) {
 				t.Fatalf("bad: %v", s)
 			}
 			if s.Node != "foo" {
@@ -702,45 +724,36 @@ func TestSession_Renew(t *testing.T) {
 }
 
 func TestSession_Renew_ACLDeny(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	dir1, s1 := testServerWithConfig(t, func(c *Config) {
-		c.ACLDatacenter = "dc1"
-		c.ACLMasterToken = "root"
-		c.ACLDefaultPolicy = "deny"
-		c.ACLEnforceVersion8 = false
+		c.PrimaryDatacenter = "dc1"
+		c.ACLsEnabled = true
+		c.ACLInitialManagementToken = "root"
+		c.ACLResolverSettings.ACLDefaultPolicy = "deny"
 	})
 	defer os.RemoveAll(dir1)
 	defer s1.Shutdown()
-	testrpc.WaitForTestAgent(t, s1.RPC, "dc1")
+
 	codec := rpcClient(t, s1)
 	defer codec.Close()
 
-	// Create the ACL.
-	req := structs.ACLRequest{
-		Datacenter: "dc1",
-		Op:         structs.ACLSet,
-		ACL: structs.ACL{
-			Name: "User token",
-			Type: structs.ACLTypeClient,
-			Rules: `
+	testrpc.WaitForTestAgent(t, s1.RPC, "dc1", testrpc.WithToken("root"))
+
+	rules := `
 session "foo" {
 	policy = "write"
 }
-`,
-		},
-		WriteRequest: structs.WriteRequest{Token: "root"},
-	}
-
-	var token string
-	if err := msgpackrpc.CallWithCodec(codec, "ACL.Apply", &req, &token); err != nil {
-		t.Fatalf("err: %v", err)
-	}
+`
+	token := createToken(t, codec, rules)
 
 	// Just add a node.
 	s1.fsm.State().EnsureNode(1, &structs.Node{Node: "foo", Address: "127.0.0.1"})
 
-	// Create a session. The token won't matter here since we don't have
-	// version 8 ACL enforcement on yet.
+	// Create a session.
 	arg := structs.SessionRequest{
 		Datacenter: "dc1",
 		Op:         structs.SessionCreate,
@@ -748,25 +761,19 @@ session "foo" {
 			Node: "foo",
 			Name: "my-session",
 		},
+		WriteRequest: structs.WriteRequest{Token: token},
 	}
 	var id string
 	if err := msgpackrpc.CallWithCodec(codec, "Session.Apply", &arg, &id); err != nil {
 		t.Fatalf("err: %v", err)
 	}
 
-	// Renew without a token should go through without version 8 ACL
-	// enforcement.
+	// Renew without a token should be rejected.
 	renewR := structs.SessionSpecificRequest{
 		Datacenter: "dc1",
-		Session:    id,
+		SessionID:  id,
 	}
 	var session structs.IndexedSessions
-	if err := msgpackrpc.CallWithCodec(codec, "Session.Renew", &renewR, &session); err != nil {
-		t.Fatalf("err: %v", err)
-	}
-
-	// Now turn on version 8 enforcement and the renew should be rejected.
-	s1.config.ACLEnforceVersion8 = true
 	err := msgpackrpc.CallWithCodec(codec, "Session.Renew", &renewR, &session)
 	if !acl.IsErrPermissionDenied(err) {
 		t.Fatalf("err: %v", err)
@@ -779,11 +786,77 @@ session "foo" {
 	}
 }
 
+func TestSession_Renew_Compat(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
+	// This method is timing sensitive, disable Parallel
+	//t.Parallel()
+	ttl := 5 * time.Second
+	TTL := ttl.String()
+
+	dir1, s1 := testServerWithConfig(t, func(c *Config) {
+		c.SessionTTLMin = ttl
+	})
+	defer os.RemoveAll(dir1)
+	defer s1.Shutdown()
+	testrpc.WaitForTestAgent(t, s1.RPC, "dc1")
+
+	codec := rpcClient(t, s1)
+	defer codec.Close()
+
+	s1.fsm.State().EnsureNode(1, &structs.Node{Node: "foo", Address: "127.0.0.1"})
+	var id string
+	arg := structs.SessionRequest{
+		Datacenter: "dc1",
+		Op:         structs.SessionCreate,
+		Session: structs.Session{
+			Node: "foo",
+			TTL:  TTL,
+		},
+	}
+	if err := msgpackrpc.CallWithCodec(codec, "Session.Apply", &arg, &id); err != nil {
+		t.Fatalf("err: %v", err)
+	}
+
+	// renew the session
+	renewR := structs.SessionSpecificRequest{
+		Datacenter: "dc1",
+		// this will get ranslated internally to the SessionID field
+		Session: id,
+	}
+	var session structs.IndexedSessions
+	if err := msgpackrpc.CallWithCodec(codec, "Session.Renew", &renewR, &session); err != nil {
+		t.Fatalf("err: %v", err)
+	}
+
+	if session.Index == 0 {
+		t.Fatalf("Bad: %v", session)
+	}
+	if len(session.Sessions) != 1 {
+		t.Fatalf("Bad: %v", session.Sessions)
+	}
+
+	s := session.Sessions[0]
+	if id != s.ID {
+		t.Fatalf("bad: %v", s)
+	}
+	if s.Node != "foo" {
+		t.Fatalf("bad: %v", s)
+	}
+}
+
 func TestSession_NodeSessions(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	dir1, s1 := testServer(t)
 	defer os.RemoveAll(dir1)
 	defer s1.Shutdown()
+
 	codec := rpcClient(t, s1)
 	defer codec.Close()
 
@@ -829,7 +902,7 @@ func TestSession_NodeSessions(t *testing.T) {
 	}
 	for i := 0; i < len(sessions.Sessions); i++ {
 		s := sessions.Sessions[i]
-		if !lib.StrContains(ids, s.ID) {
+		if !stringslice.Contains(ids, s.ID) {
 			t.Fatalf("bad: %v", s)
 		}
 		if s.Node != "foo" {
@@ -839,10 +912,15 @@ func TestSession_NodeSessions(t *testing.T) {
 }
 
 func TestSession_Apply_BadTTL(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	dir1, s1 := testServer(t)
 	defer os.RemoveAll(dir1)
 	defer s1.Shutdown()
+
 	codec := rpcClient(t, s1)
 	defer codec.Close()
 
@@ -865,7 +943,7 @@ func TestSession_Apply_BadTTL(t *testing.T) {
 	if err == nil {
 		t.Fatal("expected error")
 	}
-	if err.Error() != "Session TTL '10z' invalid: time: unknown unit z in duration 10z" {
+	if err.Error() != `Session TTL '10z' invalid: time: unknown unit "z" in duration "10z"` {
 		t.Fatalf("incorrect error message: %s", err.Error())
 	}
 
