@@ -1,6 +1,10 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: BUSL-1.1
+
 package state
 
 import (
+	"fmt"
 	"strings"
 
 	memdb "github.com/hashicorp/go-memdb"
@@ -8,9 +12,23 @@ import (
 	"github.com/hashicorp/consul/acl"
 	"github.com/hashicorp/consul/agent/consul/stream"
 	"github.com/hashicorp/consul/agent/structs"
-	"github.com/hashicorp/consul/proto/pbsubscribe"
+	"github.com/hashicorp/consul/proto/private/pbcommon"
+	"github.com/hashicorp/consul/proto/private/pbservice"
+	"github.com/hashicorp/consul/proto/private/pbsubscribe"
 )
 
+// EventSubjectService is a stream.Subject used to route and receive events for
+// a specific service.
+type EventSubjectService struct {
+	Key            string
+	EnterpriseMeta acl.EnterpriseMeta
+	PeerName       string
+
+	overrideKey       string
+	overrideNamespace string
+	overridePartition string
+}
+
 // EventPayloadCheckServiceNode is used as the Payload for a stream.Event to
 // indicates changes to a CheckServiceNode for service health.
 //
@@ -25,96 +43,139 @@ type EventPayloadCheckServiceNode struct {
 	// when the change event is for a sidecar or gateway.
 	overrideKey       string
 	overrideNamespace string
+	overridePartition string
 }
 
 func (e EventPayloadCheckServiceNode) HasReadPermission(authz acl.Authorizer) bool {
 	return e.Value.CanRead(authz) == acl.Allow
 }
 
-func (e EventPayloadCheckServiceNode) MatchesKey(key, namespace string) bool {
-	if key == "" && namespace == "" {
-		return true
-	}
+func (e EventPayloadCheckServiceNode) Subject() stream.Subject {
+	return EventSubjectService{
+		Key:            e.Value.Service.Service,
+		EnterpriseMeta: e.Value.Service.EnterpriseMeta,
+		PeerName:       e.Value.Service.PeerName,
 
-	if e.Value.Service == nil {
-		return false
+		overrideKey:       e.overrideKey,
+		overrideNamespace: e.overrideNamespace,
+		overridePartition: e.overridePartition,
 	}
+}
 
-	name := e.Value.Service.Service
-	if e.overrideKey != "" {
-		name = e.overrideKey
+func (e EventPayloadCheckServiceNode) ToSubscriptionEvent(idx uint64) *pbsubscribe.Event {
+	return &pbsubscribe.Event{
+		Index: idx,
+		Payload: &pbsubscribe.Event_ServiceHealth{
+			ServiceHealth: &pbsubscribe.ServiceHealthUpdate{
+				Op:               e.Op,
+				CheckServiceNode: pbservice.NewCheckServiceNodeFromStructs(e.Value),
+			},
+		},
 	}
-	ns := e.Value.Service.EnterpriseMeta.NamespaceOrDefault()
-	if e.overrideNamespace != "" {
-		ns = e.overrideNamespace
+}
+
+// EventPayloadServiceListUpdate is used as the Payload for a stream.Event when
+// services (not service instances) are registered/deregistered. These events
+// are used to materialize the list of services in a datacenter.
+type EventPayloadServiceListUpdate struct {
+	Op pbsubscribe.CatalogOp
+
+	Name           string
+	EnterpriseMeta acl.EnterpriseMeta
+	PeerName       string
+}
+
+func (e *EventPayloadServiceListUpdate) ToSubscriptionEvent(idx uint64) *pbsubscribe.Event {
+	return &pbsubscribe.Event{
+		Index: idx,
+		Payload: &pbsubscribe.Event_Service{
+			Service: &pbsubscribe.ServiceListUpdate{
+				Op:             e.Op,
+				Name:           e.Name,
+				EnterpriseMeta: pbcommon.NewEnterpriseMetaFromStructs(e.EnterpriseMeta),
+				PeerName:       e.PeerName,
+			},
+		},
 	}
-	return (key == "" || strings.EqualFold(key, name)) &&
-		(namespace == "" || strings.EqualFold(namespace, ns))
+}
+
+func (e *EventPayloadServiceListUpdate) Subject() stream.Subject { return stream.SubjectNone }
+
+func (e *EventPayloadServiceListUpdate) HasReadPermission(authz acl.Authorizer) bool {
+	var authzContext acl.AuthorizerContext
+	e.EnterpriseMeta.FillAuthzContext(&authzContext)
+	return authz.ServiceRead(e.Name, &authzContext) == acl.Allow
 }
 
 // serviceHealthSnapshot returns a stream.SnapshotFunc that provides a snapshot
 // of stream.Events that describe the current state of a service health query.
-func serviceHealthSnapshot(db ReadDB, topic stream.Topic) stream.SnapshotFunc {
-	return func(req stream.SubscribeRequest, buf stream.SnapshotAppender) (index uint64, err error) {
-		tx := db.ReadTxn()
-		defer tx.Abort()
+func (s *Store) ServiceHealthSnapshot(req stream.SubscribeRequest, buf stream.SnapshotAppender) (index uint64, err error) {
+	tx := s.db.ReadTxn()
+	defer tx.Abort()
 
-		connect := topic == topicServiceHealthConnect
-		entMeta := structs.NewEnterpriseMeta(req.Namespace)
-		idx, nodes, err := checkServiceNodesTxn(tx, nil, req.Key, connect, &entMeta)
-		if err != nil {
-			return 0, err
-		}
+	connect := req.Topic == EventTopicServiceHealthConnect
 
-		for i := range nodes {
-			n := nodes[i]
-			event := stream.Event{
-				Index: idx,
-				Topic: topic,
-				Payload: EventPayloadCheckServiceNode{
-					Op:    pbsubscribe.CatalogOp_Register,
-					Value: &n,
-				},
-			}
-
-			if !connect {
-				// append each event as a separate item so that they can be serialized
-				// separately, to prevent the encoding of one massive message.
-				buf.Append([]stream.Event{event})
-				continue
-			}
-
-			events, err := connectEventsByServiceKind(tx, event)
-			if err != nil {
-				return idx, err
-			}
-			buf.Append(events)
-		}
-
-		return idx, err
+	subject, ok := req.Subject.(EventSubjectService)
+	if !ok {
+		return 0, fmt.Errorf("expected SubscribeRequest.Subject to be a: state.EventSubjectService, was a: %T", req.Subject)
 	}
+
+	idx, nodes, err := checkServiceNodesTxn(tx, nil, subject.Key, connect, &subject.EnterpriseMeta, subject.PeerName)
+	if err != nil {
+		return 0, err
+	}
+
+	for i := range nodes {
+		n := nodes[i]
+		event := stream.Event{
+			Index: idx,
+			Topic: req.Topic,
+			Payload: EventPayloadCheckServiceNode{
+				Op:    pbsubscribe.CatalogOp_Register,
+				Value: &n,
+			},
+		}
+
+		if !connect {
+			// append each event as a separate item so that they can be serialized
+			// separately, to prevent the encoding of one massive message.
+			buf.Append([]stream.Event{event})
+			continue
+		}
+
+		events, err := connectEventsByServiceKind(tx, event)
+		if err != nil {
+			return idx, err
+		}
+		buf.Append(events)
+	}
+
+	return idx, err
 }
 
 // TODO: this could use NodeServiceQuery
 type nodeServiceTuple struct {
 	Node      string
 	ServiceID string
-	EntMeta   structs.EnterpriseMeta
+	EntMeta   acl.EnterpriseMeta
+	PeerName  string
 }
 
 func newNodeServiceTupleFromServiceNode(sn *structs.ServiceNode) nodeServiceTuple {
 	return nodeServiceTuple{
-		Node:      sn.Node,
+		Node:      strings.ToLower(sn.Node),
 		ServiceID: sn.ServiceID,
 		EntMeta:   sn.EnterpriseMeta,
+		PeerName:  sn.PeerName,
 	}
 }
 
 func newNodeServiceTupleFromServiceHealthCheck(hc *structs.HealthCheck) nodeServiceTuple {
 	return nodeServiceTuple{
-		Node:      hc.Node,
+		Node:      strings.ToLower(hc.Node),
 		ServiceID: hc.ServiceID,
 		EntMeta:   hc.EnterpriseMeta,
+		PeerName:  hc.PeerName,
 	}
 }
 
@@ -123,20 +184,85 @@ type serviceChange struct {
 	change     memdb.Change
 }
 
+type nodeTuple struct {
+	Node      string
+	Partition string
+	PeerName  string
+}
+
 var serviceChangeIndirect = serviceChange{changeType: changeIndirect}
 
+// ServiceListUpdateEventsFromChanges returns events representing changes to
+// the list of services from the given set of state store changes.
+func ServiceListUpdateEventsFromChanges(tx ReadTxn, changes Changes) ([]stream.Event, error) {
+	var events []stream.Event
+	for _, change := range changes.Changes {
+		if change.Table != tableKindServiceNames {
+			continue
+		}
+
+		kindName := changeObject(change).(*KindServiceName)
+
+		// TODO(peering): make this peer-aware.
+		payload := &EventPayloadServiceListUpdate{
+			Name:           kindName.Service.Name,
+			EnterpriseMeta: kindName.Service.EnterpriseMeta,
+		}
+
+		if change.Deleted() {
+			payload.Op = pbsubscribe.CatalogOp_Deregister
+		} else {
+			payload.Op = pbsubscribe.CatalogOp_Register
+		}
+
+		events = append(events, stream.Event{
+			Topic:   EventTopicServiceList,
+			Index:   changes.Index,
+			Payload: payload,
+		})
+	}
+	return events, nil
+}
+
+// ServiceListSnapshot is a stream.SnapshotFunc that returns a snapshot of
+// all service names.
+func (s *Store) ServiceListSnapshot(_ stream.SubscribeRequest, buf stream.SnapshotAppender) (uint64, error) {
+	index, names, err := s.ServiceNamesOfKind(nil, "")
+	if err != nil {
+		return 0, err
+	}
+
+	if l := len(names); l > 0 {
+		events := make([]stream.Event, l)
+		for idx, name := range names {
+			events[idx] = stream.Event{
+				Topic: EventTopicServiceList,
+				Index: index,
+				Payload: &EventPayloadServiceListUpdate{
+					Op:             pbsubscribe.CatalogOp_Register,
+					Name:           name.Service.Name,
+					EnterpriseMeta: name.Service.EnterpriseMeta,
+				},
+			}
+		}
+		buf.Append(events)
+	}
+
+	return index, nil
+}
+
 // ServiceHealthEventsFromChanges returns all the service and Connect health
 // events that should be emitted given a set of changes to the state store.
 func ServiceHealthEventsFromChanges(tx ReadTxn, changes Changes) ([]stream.Event, error) {
 	var events []stream.Event
 
-	var nodeChanges map[string]changeType
+	var nodeChanges map[nodeTuple]changeType
 	var serviceChanges map[nodeServiceTuple]serviceChange
 	var termGatewayChanges map[structs.ServiceName]map[structs.ServiceName]serviceChange
 
-	markNode := func(node string, typ changeType) {
+	markNode := func(node nodeTuple, typ changeType) {
 		if nodeChanges == nil {
-			nodeChanges = make(map[string]changeType)
+			nodeChanges = make(map[nodeTuple]changeType)
 		}
 		// If the caller has an actual node mutation ensure we store it even if the
 		// node is already marked. If the caller is just marking the node dirty
@@ -161,14 +287,15 @@ func ServiceHealthEventsFromChanges(tx ReadTxn, changes Changes) ([]stream.Event
 
 	for _, change := range changes.Changes {
 		switch change.Table {
-		case "nodes":
+		case tableNodes:
 			// Node changed in some way, if it's not a delete, we'll need to
 			// re-deliver CheckServiceNode results for all services on that node but
 			// we mark it anyway because if it _is_ a delete then we need to know that
 			// later to avoid trying to deliver events when node level checks mark the
 			// node as "changed".
 			n := changeObject(change).(*structs.Node)
-			markNode(n.Node, changeTypeFromChange(change))
+			tuple := newNodeTupleFromNode(n)
+			markNode(tuple, changeTypeFromChange(change))
 
 		case tableServices:
 			sn := changeObject(change).(*structs.ServiceNode)
@@ -187,7 +314,8 @@ func ServiceHealthEventsFromChanges(tx ReadTxn, changes Changes) ([]stream.Event
 				after := change.After.(*structs.HealthCheck)
 				if after.ServiceID == "" || before.ServiceID == "" {
 					// check before and/or after is node-scoped
-					markNode(after.Node, changeIndirect)
+					nt := newNodeTupleFromHealthCheck(after)
+					markNode(nt, changeIndirect)
 				} else {
 					// Check changed which means we just need to emit for the linked
 					// service.
@@ -206,7 +334,8 @@ func ServiceHealthEventsFromChanges(tx ReadTxn, changes Changes) ([]stream.Event
 				obj := changeObject(change).(*structs.HealthCheck)
 				if obj.ServiceID == "" {
 					// Node level check
-					markNode(obj.Node, changeIndirect)
+					nt := newNodeTupleFromHealthCheck(obj)
+					markNode(nt, changeIndirect)
 				} else {
 					markService(newNodeServiceTupleFromServiceHealthCheck(obj), serviceChangeIndirect)
 				}
@@ -250,7 +379,8 @@ func ServiceHealthEventsFromChanges(tx ReadTxn, changes Changes) ([]stream.Event
 			continue
 		}
 		// Rebuild events for all services on this node
-		es, err := newServiceHealthEventsForNode(tx, changes.Index, node)
+		es, err := newServiceHealthEventsForNode(tx, changes.Index, node.Node,
+			structs.WildcardEnterpriseMetaInPartition(node.Partition), node.PeerName)
 		if err != nil {
 			return nil, err
 		}
@@ -286,7 +416,7 @@ func ServiceHealthEventsFromChanges(tx ReadTxn, changes Changes) ([]stream.Event
 			}
 		}
 
-		if _, ok := nodeChanges[tuple.Node]; ok {
+		if _, ok := nodeChanges[tuple.nodeTuple()]; ok {
 			// We already rebuilt events for everything on this node, no need to send
 			// a duplicate.
 			continue
@@ -299,11 +429,15 @@ func ServiceHealthEventsFromChanges(tx ReadTxn, changes Changes) ([]stream.Event
 		events = append(events, e)
 	}
 
-	for gatewayName, serviceChanges := range termGatewayChanges {
-		for serviceName, gsChange := range serviceChanges {
+	for gatewayName, svcChanges := range termGatewayChanges {
+		for serviceName, gsChange := range svcChanges {
 			gs := changeObject(gsChange.change).(*structs.GatewayService)
 
-			q := Query{Value: gs.Gateway.Name, EnterpriseMeta: gatewayName.EnterpriseMeta}
+			q := Query{
+				Value:          gs.Gateway.Name,
+				EnterpriseMeta: gatewayName.EnterpriseMeta,
+				PeerName:       structs.TODOPeerKeyword,
+			}
 			_, nodes, err := serviceNodesTxn(tx, nil, indexService, q)
 			if err != nil {
 				return nil, err
@@ -314,12 +448,15 @@ func ServiceHealthEventsFromChanges(tx ReadTxn, changes Changes) ([]stream.Event
 				for _, sn := range nodes {
 					e := newServiceHealthEventDeregister(changes.Index, sn)
 
-					e.Topic = topicServiceHealthConnect
+					e.Topic = EventTopicServiceHealthConnect
 					payload := e.Payload.(EventPayloadCheckServiceNode)
 					payload.overrideKey = serviceName.Name
 					if gatewayName.EnterpriseMeta.NamespaceOrDefault() != serviceName.EnterpriseMeta.NamespaceOrDefault() {
 						payload.overrideNamespace = serviceName.EnterpriseMeta.NamespaceOrDefault()
 					}
+					if gatewayName.EnterpriseMeta.PartitionOrDefault() != serviceName.EnterpriseMeta.PartitionOrDefault() {
+						payload.overridePartition = serviceName.EnterpriseMeta.PartitionOrDefault()
+					}
 					e.Payload = payload
 
 					events = append(events, e)
@@ -333,17 +470,26 @@ func ServiceHealthEventsFromChanges(tx ReadTxn, changes Changes) ([]stream.Event
 			// Build service events and append them
 			for _, sn := range nodes {
 				tuple := newNodeServiceTupleFromServiceNode(sn)
+
+				// If we're already sending an event for the service, don't send another.
+				if _, ok := serviceChanges[tuple]; ok {
+					continue
+				}
+
 				e, err := newServiceHealthEventForService(tx, changes.Index, tuple)
 				if err != nil {
 					return nil, err
 				}
 
-				e.Topic = topicServiceHealthConnect
+				e.Topic = EventTopicServiceHealthConnect
 				payload := e.Payload.(EventPayloadCheckServiceNode)
 				payload.overrideKey = serviceName.Name
 				if gatewayName.EnterpriseMeta.NamespaceOrDefault() != serviceName.EnterpriseMeta.NamespaceOrDefault() {
 					payload.overrideNamespace = serviceName.EnterpriseMeta.NamespaceOrDefault()
 				}
+				if gatewayName.EnterpriseMeta.PartitionOrDefault() != serviceName.EnterpriseMeta.PartitionOrDefault() {
+					payload.overridePartition = serviceName.EnterpriseMeta.PartitionOrDefault()
+				}
 				e.Payload = payload
 
 				events = append(events, e)
@@ -373,7 +519,7 @@ func isConnectProxyDestinationServiceChange(idx uint64, before, after *structs.S
 	}
 
 	e := newServiceHealthEventDeregister(idx, before)
-	e.Topic = topicServiceHealthConnect
+	e.Topic = EventTopicServiceHealthConnect
 	payload := e.Payload.(EventPayloadCheckServiceNode)
 	payload.overrideKey = payload.Value.Service.Proxy.DestinationServiceName
 	e.Payload = payload
@@ -414,7 +560,7 @@ func serviceHealthToConnectEvents(
 ) ([]stream.Event, error) {
 	var result []stream.Event
 	for _, event := range events {
-		if event.Topic != topicServiceHealth { // event.Topic == topicServiceHealthConnect
+		if event.Topic != EventTopicServiceHealth { // event.Topic == topicServiceHealthConnect
 			// Skip non-health or any events already emitted to Connect topic
 			continue
 		}
@@ -437,7 +583,7 @@ func connectEventsByServiceKind(tx ReadTxn, origEvent stream.Event) ([]stream.Ev
 	}
 
 	event := origEvent // shallow copy the event
-	event.Topic = topicServiceHealthConnect
+	event.Topic = EventTopicServiceHealthConnect
 
 	if node.Service.Connect.Native {
 		return []stream.Event{event}, nil
@@ -453,6 +599,8 @@ func connectEventsByServiceKind(tx ReadTxn, origEvent stream.Event) ([]stream.Ev
 	case structs.ServiceKindTerminatingGateway:
 		var result []stream.Event
 
+		// TODO(peering): handle terminating gateways somehow
+
 		sn := structs.ServiceName{
 			Name:           node.Service.Service,
 			EnterpriseMeta: node.Service.EnterpriseMeta,
@@ -474,12 +622,15 @@ func connectEventsByServiceKind(tx ReadTxn, origEvent stream.Event) ([]stream.Ev
 }
 
 func copyEventForService(event stream.Event, service structs.ServiceName) stream.Event {
-	event.Topic = topicServiceHealthConnect
+	event.Topic = EventTopicServiceHealthConnect
 	payload := event.Payload.(EventPayloadCheckServiceNode)
 	payload.overrideKey = service.Name
 	if payload.Value.Service.EnterpriseMeta.NamespaceOrDefault() != service.EnterpriseMeta.NamespaceOrDefault() {
 		payload.overrideNamespace = service.EnterpriseMeta.NamespaceOrDefault()
 	}
+	if payload.Value.Service.EnterpriseMeta.PartitionOrDefault() != service.EnterpriseMeta.PartitionOrDefault() {
+		payload.overridePartition = service.EnterpriseMeta.PartitionOrDefault()
+	}
 
 	event.Payload = payload
 	return event
@@ -494,16 +645,20 @@ func getPayloadCheckServiceNode(payload stream.Payload) *structs.CheckServiceNod
 }
 
 // newServiceHealthEventsForNode returns health events for all services on the
-// given node. This mirrors some of the the logic in the oddly-named
+// given node. This mirrors some of the logic in the oddly-named
 // parseCheckServiceNodes but is more efficient since we know they are all on
 // the same node.
-func newServiceHealthEventsForNode(tx ReadTxn, idx uint64, node string) ([]stream.Event, error) {
-	services, err := tx.Get(tableServices, indexNode, Query{Value: node})
+func newServiceHealthEventsForNode(tx ReadTxn, idx uint64, node string, entMeta *acl.EnterpriseMeta, peerName string) ([]stream.Event, error) {
+	services, err := tx.Get(tableServices, indexNode, Query{
+		Value:          node,
+		EnterpriseMeta: *entMeta,
+		PeerName:       peerName,
+	})
 	if err != nil {
 		return nil, err
 	}
 
-	n, checksFunc, err := getNodeAndChecks(tx, node)
+	n, checksFunc, err := getNodeAndChecks(tx, node, entMeta, peerName)
 	if err != nil {
 		return nil, err
 	}
@@ -521,9 +676,13 @@ func newServiceHealthEventsForNode(tx ReadTxn, idx uint64, node string) ([]strea
 
 // getNodeAndNodeChecks returns a the node structure and a function that returns
 // the full list of checks for a specific service on that node.
-func getNodeAndChecks(tx ReadTxn, node string) (*structs.Node, serviceChecksFunc, error) {
+func getNodeAndChecks(tx ReadTxn, node string, entMeta *acl.EnterpriseMeta, peerName string) (*structs.Node, serviceChecksFunc, error) {
 	// Fetch the node
-	nodeRaw, err := tx.First(tableNodes, indexID, Query{Value: node})
+	nodeRaw, err := tx.First(tableNodes, indexID, Query{
+		Value:          node,
+		EnterpriseMeta: *entMeta,
+		PeerName:       peerName,
+	})
 	if err != nil {
 		return nil, nil, err
 	}
@@ -532,7 +691,11 @@ func getNodeAndChecks(tx ReadTxn, node string) (*structs.Node, serviceChecksFunc
 	}
 	n := nodeRaw.(*structs.Node)
 
-	iter, err := tx.Get(tableChecks, indexNode, Query{Value: node})
+	iter, err := tx.Get(tableChecks, indexNode, Query{
+		Value:          node,
+		EnterpriseMeta: *entMeta,
+		PeerName:       peerName,
+	})
 	if err != nil {
 		return nil, nil, err
 	}
@@ -566,12 +729,17 @@ func getNodeAndChecks(tx ReadTxn, node string) (*structs.Node, serviceChecksFunc
 type serviceChecksFunc func(serviceID string) structs.HealthChecks
 
 func newServiceHealthEventForService(tx ReadTxn, idx uint64, tuple nodeServiceTuple) (stream.Event, error) {
-	n, checksFunc, err := getNodeAndChecks(tx, tuple.Node)
+	n, checksFunc, err := getNodeAndChecks(tx, tuple.Node, &tuple.EntMeta, tuple.PeerName)
 	if err != nil {
 		return stream.Event{}, err
 	}
 
-	svc, err := tx.Get(tableServices, indexID, NodeServiceQuery{EnterpriseMeta: tuple.EntMeta, Node: tuple.Node, Service: tuple.ServiceID})
+	svc, err := tx.Get(tableServices, indexID, NodeServiceQuery{
+		EnterpriseMeta: tuple.EntMeta,
+		Node:           tuple.Node,
+		Service:        tuple.ServiceID,
+		PeerName:       tuple.PeerName,
+	})
 	if err != nil {
 		return stream.Event{}, err
 	}
@@ -597,7 +765,7 @@ func newServiceHealthEventRegister(
 		Checks:  checks,
 	}
 	return stream.Event{
-		Topic: topicServiceHealth,
+		Topic: EventTopicServiceHealth,
 		Index: idx,
 		Payload: EventPayloadCheckServiceNode{
 			Op:    pbsubscribe.CatalogOp_Register,
@@ -615,15 +783,21 @@ func newServiceHealthEventDeregister(idx uint64, sn *structs.ServiceNode) stream
 	// This is also important because if the service was deleted as part of a
 	// whole node deregistering then the node record won't actually exist now
 	// anyway and we'd have to plumb it through from the changeset above.
+
+	entMeta := sn.EnterpriseMeta
+	entMeta.Normalize()
+
 	csn := &structs.CheckServiceNode{
 		Node: &structs.Node{
-			Node: sn.Node,
+			Node:      sn.Node,
+			Partition: entMeta.PartitionOrEmpty(),
+			PeerName:  sn.PeerName,
 		},
 		Service: sn.ToNodeService(),
 	}
 
 	return stream.Event{
-		Topic: topicServiceHealth,
+		Topic: EventTopicServiceHealth,
 		Index: idx,
 		Payload: EventPayloadCheckServiceNode{
 			Op:    pbsubscribe.CatalogOp_Deregister,
