@@ -1,6 +1,10 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: BUSL-1.1
+
 package local
 
 import (
+	"context"
 	"fmt"
 	"reflect"
 	"strconv"
@@ -9,16 +13,20 @@ import (
 	"sync/atomic"
 	"time"
 
-	"github.com/armon/go-metrics"
-	"github.com/armon/go-metrics/prometheus"
-	"github.com/hashicorp/go-hclog"
-
 	"github.com/hashicorp/consul/acl"
+	"github.com/hashicorp/consul/acl/resolver"
 	"github.com/hashicorp/consul/agent/structs"
 	"github.com/hashicorp/consul/agent/token"
 	"github.com/hashicorp/consul/api"
 	"github.com/hashicorp/consul/lib"
+	"github.com/hashicorp/consul/lib/stringslice"
 	"github.com/hashicorp/consul/types"
+
+	"github.com/armon/go-metrics"
+	"github.com/armon/go-metrics/prometheus"
+	"github.com/hashicorp/go-hclog"
+	"github.com/hashicorp/go-multierror"
+	"github.com/mitchellh/copystructure"
 )
 
 var StateCounters = []prometheus.CounterDefinition{
@@ -42,10 +50,6 @@ var StateCounters = []prometheus.CounterDefinition{
 		Name: []string{"acl", "blocked", "node", "registration"},
 		Help: "Increments whenever a registration fails for a node (blocked by an ACL)",
 	},
-	{
-		Name: []string{"acl", "blocked", "node", "deregistration"},
-		Help: "Increments whenever a deregistration fails for a node (blocked by an ACL)",
-	},
 }
 
 const fullSyncReadMaxStale = 2 * time.Second
@@ -58,6 +62,8 @@ type Config struct {
 	DiscardCheckOutput  bool
 	NodeID              types.NodeID
 	NodeName            string
+	NodeLocality        *structs.Locality
+	Partition           string // this defaults if empty
 	TaggedAddresses     map[string]string
 }
 
@@ -78,6 +84,10 @@ type ServiceState struct {
 	// but has not been removed on the server yet.
 	Deleted bool
 
+	// IsLocallyDefined indicates whether the service was defined locally in config
+	// as opposed to being registered through the Agent API.
+	IsLocallyDefined bool
+
 	// WatchCh is closed when the service state changes. Suitable for use in a
 	// memdb.WatchSet when watching agent local changes with hash-based blocking.
 	WatchCh chan struct{}
@@ -122,6 +132,10 @@ type CheckState struct {
 	// Deleted is true when the health check record has been marked as
 	// deleted but has not been removed on the server yet.
 	Deleted bool
+
+	// IsLocallyDefined indicates whether the check was defined locally in config
+	// as opposed to being registered through the Agent API.
+	IsLocallyDefined bool
 }
 
 // Clone returns a shallow copy of the object.
@@ -148,8 +162,8 @@ func (c *CheckState) CriticalFor() time.Duration {
 }
 
 type rpc interface {
-	RPC(method string, args interface{}, reply interface{}) error
-	ResolveTokenToIdentity(secretID string) (structs.ACLIdentity, error)
+	RPC(ctx context.Context, method string, args interface{}, reply interface{}) error
+	ResolveTokenAndDefaultMeta(token string, entMeta *acl.EnterpriseMeta, authzContext *acl.AuthorizerContext) (resolver.Result, error)
 }
 
 // State is used to represent the node's services,
@@ -176,6 +190,8 @@ type State struct {
 	// Config is the agent config
 	config Config
 
+	agentEnterpriseMeta acl.EnterpriseMeta
+
 	// nodeInfoInSync tracks whether the server has our correct top-level
 	// node information in sync
 	nodeInfoInSync bool
@@ -208,14 +224,15 @@ type State struct {
 // NewState creates a new local state for the agent.
 func NewState(c Config, logger hclog.Logger, tokens *token.Store) *State {
 	l := &State{
-		config:         c,
-		logger:         logger,
-		services:       make(map[structs.ServiceID]*ServiceState),
-		checks:         make(map[structs.CheckID]*CheckState),
-		checkAliases:   make(map[structs.ServiceID]map[structs.CheckID]chan<- struct{}),
-		metadata:       make(map[string]string),
-		tokens:         tokens,
-		notifyHandlers: make(map[chan<- struct{}]struct{}),
+		config:              c,
+		logger:              logger,
+		services:            make(map[structs.ServiceID]*ServiceState),
+		checks:              make(map[structs.CheckID]*CheckState),
+		checkAliases:        make(map[structs.ServiceID]map[structs.CheckID]chan<- struct{}),
+		metadata:            make(map[string]string),
+		tokens:              tokens,
+		notifyHandlers:      make(map[chan<- struct{}]struct{}),
+		agentEnterpriseMeta: *structs.NodeEnterpriseMetaInPartition(c.Partition),
 	}
 	l.SetDiscardCheckOutput(c.DiscardCheckOutput)
 	return l
@@ -241,50 +258,61 @@ func (l *State) ServiceToken(id structs.ServiceID) string {
 // aclTokenForServiceSync returns an ACL token associated with a service. If there is
 // no ACL token associated with the service, fallback is used to return a value.
 // This method is not synchronized and the lock must already be held.
-func (l *State) aclTokenForServiceSync(id structs.ServiceID, fallback func() string) string {
+func (l *State) aclTokenForServiceSync(id structs.ServiceID, fallbacks ...func() string) string {
 	if s := l.services[id]; s != nil && s.Token != "" {
 		return s.Token
 	}
-	return fallback()
+	for _, fb := range fallbacks {
+		if tok := fb(); tok != "" {
+			return tok
+		}
+	}
+	return ""
 }
 
-// AddService is used to add a service entry to the local state.
-// This entry is persistent and the agent will make a best effort to
-// ensure it is registered
-func (l *State) AddService(service *structs.NodeService, token string) error {
-	l.Lock()
-	defer l.Unlock()
-	return l.addServiceLocked(service, token)
-}
-
-func (l *State) addServiceLocked(service *structs.NodeService, token string) error {
+func (l *State) addServiceLocked(service *structs.NodeService, token string, isLocal bool) error {
 	if service == nil {
 		return fmt.Errorf("no service")
 	}
 
+	// Avoid having the stored service have any call-site ownership.
+	var err error
+	service, err = cloneService(service)
+	if err != nil {
+		return err
+	}
+
 	// use the service name as id if the id was omitted
 	if service.ID == "" {
 		service.ID = service.Service
 	}
 
+	if l.agentEnterpriseMeta.PartitionOrDefault() != service.PartitionOrDefault() {
+		return fmt.Errorf("cannot add service ID %q to node in partition %q", service.CompoundServiceID(), l.config.Partition)
+	}
+
 	l.setServiceStateLocked(&ServiceState{
-		Service: service,
-		Token:   token,
+		Service:          service,
+		Token:            token,
+		IsLocallyDefined: isLocal,
 	})
 	return nil
 }
 
-// AddServiceWithChecks adds a service and its check tp the local state atomically
-func (l *State) AddServiceWithChecks(service *structs.NodeService, checks []*structs.HealthCheck, token string) error {
+// AddServiceWithChecks adds a service entry and its checks to the local state
+// atomically This entry is persistent and the agent will make a best effort to
+// ensure it is registered. The isLocallyDefined parameter indicates whether
+// the service and checks are sourced from local agent configuration files.
+func (l *State) AddServiceWithChecks(service *structs.NodeService, checks []*structs.HealthCheck, token string, isLocallyDefined bool) error {
 	l.Lock()
 	defer l.Unlock()
 
-	if err := l.addServiceLocked(service, token); err != nil {
+	if err := l.addServiceLocked(service, token, isLocallyDefined); err != nil {
 		return err
 	}
 
 	for _, check := range checks {
-		if err := l.addCheckLocked(check, token); err != nil {
+		if err := l.addCheckLocked(check, token, isLocallyDefined); err != nil {
 			return err
 		}
 	}
@@ -320,7 +348,14 @@ func (l *State) RemoveServiceWithChecks(serviceID structs.ServiceID, checkIDs []
 func (l *State) removeServiceLocked(id structs.ServiceID) error {
 	s := l.services[id]
 	if s == nil || s.Deleted {
-		return fmt.Errorf("Service %q does not exist", id)
+		// Take care if modifying this error message.
+		// deleteService assumes the Catalog.Deregister RPC call will include "Unknown service"
+		// in the error if deregistration fails due to a service with that ID not existing.
+
+		// When the service register endpoint is called, this error message is also typically
+		// shadowed by vetServiceUpdateWithAuthorizer, which checks for the existence of the
+		// service and, if none is found, returns an error before this function is ever called.
+		return fmt.Errorf("Unknown service ID %q. Ensure that the service ID is passed, not the service name.", id)
 	}
 
 	// To remove the service on the server we need the token.
@@ -340,8 +375,8 @@ func (l *State) removeServiceLocked(id structs.ServiceID) error {
 	return nil
 }
 
-// Service returns the locally registered service that the
-// agent is aware of and are being kept in sync with the server
+// Service returns the locally registered service that the agent is aware of
+// with this ID and are being kept in sync with the server.
 func (l *State) Service(id structs.ServiceID) *structs.NodeService {
 	l.RLock()
 	defer l.RUnlock()
@@ -353,9 +388,43 @@ func (l *State) Service(id structs.ServiceID) *structs.NodeService {
 	return s.Service
 }
 
-// Services returns the locally registered services that the
+// ServicesByName returns all the locally registered service instances that the
+// agent is aware of with this name and are being kept in sync with the server
+func (l *State) ServicesByName(sn structs.ServiceName) []*structs.NodeService {
+	l.RLock()
+	defer l.RUnlock()
+
+	var found []*structs.NodeService
+	for id, s := range l.services {
+		if s.Deleted {
+			continue
+		}
+
+		if !sn.EnterpriseMeta.Matches(&id.EnterpriseMeta) {
+			continue
+		}
+		if s.Service.Service == sn.Name {
+			found = append(found, s.Service)
+		}
+	}
+	return found
+}
+
+// AllServices returns the locally registered services that the
 // agent is aware of and are being kept in sync with the server
-func (l *State) Services(entMeta *structs.EnterpriseMeta) map[structs.ServiceID]*structs.NodeService {
+func (l *State) AllServices() map[structs.ServiceID]*structs.NodeService {
+	return l.listServices(false, nil)
+}
+
+// Services returns the locally registered services that the agent is aware of
+// and are being kept in sync with the server
+//
+// Results are scoped to the provided namespace and partition.
+func (l *State) Services(entMeta *acl.EnterpriseMeta) map[structs.ServiceID]*structs.NodeService {
+	return l.listServices(true, entMeta)
+}
+
+func (l *State) listServices(filtered bool, entMeta *acl.EnterpriseMeta) map[structs.ServiceID]*structs.NodeService {
 	l.RLock()
 	defer l.RUnlock()
 
@@ -365,7 +434,7 @@ func (l *State) Services(entMeta *structs.EnterpriseMeta) map[structs.ServiceID]
 			continue
 		}
 
-		if !entMeta.Matches(&id.EnterpriseMeta) {
+		if filtered && !entMeta.Matches(&id.EnterpriseMeta) {
 			continue
 		}
 		m[id] = s.Service
@@ -395,6 +464,10 @@ func (l *State) SetServiceState(s *ServiceState) {
 	l.Lock()
 	defer l.Unlock()
 
+	if l.agentEnterpriseMeta.PartitionOrDefault() != s.Service.PartitionOrDefault() {
+		return
+	}
+
 	l.setServiceStateLocked(s)
 }
 
@@ -423,7 +496,7 @@ func (l *State) setServiceStateLocked(s *ServiceState) {
 // ServiceStates returns a shallow copy of all service state records.
 // The service record still points to the original service record and
 // must not be modified.
-func (l *State) ServiceStates(entMeta *structs.EnterpriseMeta) map[structs.ServiceID]*ServiceState {
+func (l *State) ServiceStates(entMeta *acl.EnterpriseMeta) map[structs.ServiceID]*ServiceState {
 	l.RLock()
 	defer l.RUnlock()
 
@@ -454,47 +527,62 @@ func (l *State) CheckToken(id structs.CheckID) string {
 // aclTokenForCheckSync returns an ACL token associated with a check. If there is
 // no ACL token associated with the check, the callback is used to return a value.
 // This method is not synchronized and the lock must already be held.
-func (l *State) aclTokenForCheckSync(id structs.CheckID, fallback func() string) string {
+func (l *State) aclTokenForCheckSync(id structs.CheckID, fallbacks ...func() string) string {
 	if c := l.checks[id]; c != nil && c.Token != "" {
 		return c.Token
 	}
-	return fallback()
+	for _, fb := range fallbacks {
+		if tok := fb(); tok != "" {
+			return tok
+		}
+	}
+	return ""
 }
 
-// AddCheck is used to add a health check to the local state.
-// This entry is persistent and the agent will make a best effort to
-// ensure it is registered
-func (l *State) AddCheck(check *structs.HealthCheck, token string) error {
+// AddCheck is used to add a health check to the local state. This entry is
+// persistent and the agent will make a best effort to ensure it is registered.
+// The isLocallyDefined parameter indicates whether the checks are sourced from
+// local agent configuration files.
+func (l *State) AddCheck(check *structs.HealthCheck, token string, isLocallyDefined bool) error {
 	l.Lock()
 	defer l.Unlock()
 
-	return l.addCheckLocked(check, token)
+	return l.addCheckLocked(check, token, isLocallyDefined)
 }
 
-func (l *State) addCheckLocked(check *structs.HealthCheck, token string) error {
+func (l *State) addCheckLocked(check *structs.HealthCheck, token string, isLocal bool) error {
 	if check == nil {
 		return fmt.Errorf("no check")
 	}
 
-	// clone the check since we will be modifying it.
-	check = check.Clone()
+	// Avoid having the stored check have any call-site ownership.
+	var err error
+	check, err = cloneCheck(check)
+	if err != nil {
+		return err
+	}
 
 	if l.discardCheckOutput.Load().(bool) {
 		check.Output = ""
 	}
 
+	// hard-set the node name and partition
+	check.Node = l.config.NodeName
+	check.EnterpriseMeta = acl.NewEnterpriseMetaWithPartition(
+		l.agentEnterpriseMeta.PartitionOrEmpty(),
+		check.NamespaceOrEmpty(),
+	)
+
 	// if there is a serviceID associated with the check, make sure it exists before adding it
 	// NOTE - This logic may be moved to be handled within the Agent's Addcheck method after a refactor
 	if _, ok := l.services[check.CompoundServiceID()]; check.ServiceID != "" && !ok {
-		return fmt.Errorf("Check %q refers to non-existent service %q", check.CheckID, check.ServiceID)
+		return fmt.Errorf("Check ID %q refers to non-existent service ID %q", check.CheckID, check.ServiceID)
 	}
 
-	// hard-set the node name
-	check.Node = l.config.NodeName
-
 	l.setCheckStateLocked(&CheckState{
-		Check: check,
-		Token: token,
+		Check:            check,
+		Token:            token,
+		IsLocallyDefined: isLocal,
 	})
 	return nil
 }
@@ -510,6 +598,13 @@ func (l *State) AddAliasCheck(checkID structs.CheckID, srcServiceID structs.Serv
 	l.Lock()
 	defer l.Unlock()
 
+	if l.agentEnterpriseMeta.PartitionOrDefault() != checkID.PartitionOrDefault() {
+		return fmt.Errorf("cannot add alias check ID %q to node in partition %q", checkID.String(), l.config.Partition)
+	}
+	if l.agentEnterpriseMeta.PartitionOrDefault() != srcServiceID.PartitionOrDefault() {
+		return fmt.Errorf("cannot add alias check for %q to node in partition %q", srcServiceID.String(), l.config.Partition)
+	}
+
 	m, ok := l.checkAliases[srcServiceID]
 	if !ok {
 		m = make(map[structs.CheckID]chan<- struct{})
@@ -555,7 +650,7 @@ func (l *State) RemoveCheck(id structs.CheckID) error {
 func (l *State) removeCheckLocked(id structs.CheckID) error {
 	c := l.checks[id]
 	if c == nil || c.Deleted {
-		return fmt.Errorf("Check %q does not exist", id)
+		return fmt.Errorf("Check ID %q does not exist", id)
 	}
 
 	// If this is a check for an aliased service, then notify the waiters.
@@ -663,11 +758,23 @@ func (l *State) Check(id structs.CheckID) *structs.HealthCheck {
 	return c.Check
 }
 
+// AllChecks returns the locally registered checks that the
+// agent is aware of and are being kept in sync with the server
+func (l *State) AllChecks() map[structs.CheckID]*structs.HealthCheck {
+	return l.listChecks(false, nil)
+}
+
 // Checks returns the locally registered checks that the
 // agent is aware of and are being kept in sync with the server
-func (l *State) Checks(entMeta *structs.EnterpriseMeta) map[structs.CheckID]*structs.HealthCheck {
+//
+// Results are scoped to the provided namespace and partition.
+func (l *State) Checks(entMeta *acl.EnterpriseMeta) map[structs.CheckID]*structs.HealthCheck {
+	return l.listChecks(true, entMeta)
+}
+
+func (l *State) listChecks(filtered bool, entMeta *acl.EnterpriseMeta) map[structs.CheckID]*structs.HealthCheck {
 	m := make(map[structs.CheckID]*structs.HealthCheck)
-	for id, c := range l.CheckStates(entMeta) {
+	for id, c := range l.listCheckStates(filtered, entMeta) {
 		m[id] = c.Check
 	}
 	return m
@@ -719,6 +826,10 @@ func (l *State) SetCheckState(c *CheckState) {
 	l.Lock()
 	defer l.Unlock()
 
+	if l.agentEnterpriseMeta.PartitionOrDefault() != c.Check.PartitionOrDefault() {
+		return
+	}
+
 	l.setCheckStateLocked(c)
 }
 
@@ -727,6 +838,12 @@ func (l *State) setCheckStateLocked(c *CheckState) {
 	existing := l.checks[id]
 	if existing != nil {
 		c.InSync = c.Check.IsSame(existing.Check)
+		// If the existing check has a Defercheck, it needs to be
+		// assigned to the new check
+		if existing.DeferCheck != nil && c.DeferCheck == nil {
+			c.DeferCheck = existing.DeferCheck
+			c.InSync = false
+		}
 	}
 
 	l.checks[id] = c
@@ -737,11 +854,25 @@ func (l *State) setCheckStateLocked(c *CheckState) {
 	l.TriggerSyncChanges()
 }
 
+// AllCheckStates returns a shallow copy of all health check state records.
+// The map contains a shallow copy of the current check states.
+//
+// The defer timers still point to the original values and must not be modified.
+func (l *State) AllCheckStates() map[structs.CheckID]*CheckState {
+	return l.listCheckStates(false, nil)
+}
+
 // CheckStates returns a shallow copy of all health check state records.
 // The map contains a shallow copy of the current check states.
 //
 // The defer timers still point to the original values and must not be modified.
-func (l *State) CheckStates(entMeta *structs.EnterpriseMeta) map[structs.CheckID]*CheckState {
+//
+// Results are scoped to the provided namespace and partition.
+func (l *State) CheckStates(entMeta *acl.EnterpriseMeta) map[structs.CheckID]*CheckState {
+	return l.listCheckStates(true, entMeta)
+}
+
+func (l *State) listCheckStates(filtered bool, entMeta *acl.EnterpriseMeta) map[structs.CheckID]*CheckState {
 	l.RLock()
 	defer l.RUnlock()
 
@@ -750,7 +881,7 @@ func (l *State) CheckStates(entMeta *structs.EnterpriseMeta) map[structs.CheckID
 		if c.Deleted {
 			continue
 		}
-		if !entMeta.Matches(&id.EnterpriseMeta) {
+		if filtered && !entMeta.Matches(&id.EnterpriseMeta) {
 			continue
 		}
 		m[id] = c.Clone()
@@ -758,12 +889,27 @@ func (l *State) CheckStates(entMeta *structs.EnterpriseMeta) map[structs.CheckID
 	return m
 }
 
+// AllCriticalCheckStates returns the locally registered checks that the
+// agent is aware of and are being kept in sync with the server.
+// The map contains a shallow copy of the current check states.
+//
+// The defer timers still point to the original values and must not be modified.
+func (l *State) AllCriticalCheckStates() map[structs.CheckID]*CheckState {
+	return l.listCriticalCheckStates(false, nil)
+}
+
 // CriticalCheckStates returns the locally registered checks that the
 // agent is aware of and are being kept in sync with the server.
 // The map contains a shallow copy of the current check states.
 //
 // The defer timers still point to the original values and must not be modified.
-func (l *State) CriticalCheckStates(entMeta *structs.EnterpriseMeta) map[structs.CheckID]*CheckState {
+//
+// Results are scoped to the provided namespace and partition.
+func (l *State) CriticalCheckStates(entMeta *acl.EnterpriseMeta) map[structs.CheckID]*CheckState {
+	return l.listCriticalCheckStates(true, entMeta)
+}
+
+func (l *State) listCriticalCheckStates(filtered bool, entMeta *acl.EnterpriseMeta) map[structs.CheckID]*CheckState {
 	l.RLock()
 	defer l.RUnlock()
 
@@ -772,7 +918,7 @@ func (l *State) CriticalCheckStates(entMeta *structs.EnterpriseMeta) map[structs
 		if c.Deleted || !c.Critical() {
 			continue
 		}
-		if !entMeta.Matches(&id.EnterpriseMeta) {
+		if filtered && !entMeta.Matches(&id.EnterpriseMeta) {
 			continue
 		}
 		m[id] = c.Clone()
@@ -887,14 +1033,14 @@ func (l *State) updateSyncState() error {
 			AllowStale:       true,
 			MaxStaleDuration: fullSyncReadMaxStale,
 		},
-		EnterpriseMeta: *structs.WildcardEnterpriseMeta(),
+		EnterpriseMeta: *l.agentEnterpriseMeta.WithWildcardNamespace(),
 	}
 
 	var out1 structs.IndexedNodeServiceList
 	remoteServices := make(map[structs.ServiceID]*structs.NodeService)
 	var svcNode *structs.Node
 
-	if err := l.Delegate.RPC("Catalog.NodeServiceList", &req, &out1); err == nil {
+	if err := l.Delegate.RPC(context.Background(), "Catalog.NodeServiceList", &req, &out1); err == nil {
 		for _, svc := range out1.NodeServices.Services {
 			remoteServices[svc.CompoundServiceID()] = svc
 		}
@@ -903,7 +1049,7 @@ func (l *State) updateSyncState() error {
 	} else if errMsg := err.Error(); strings.Contains(errMsg, "rpc: can't find method") {
 		// fallback to the old RPC
 		var out1 structs.IndexedNodeServices
-		if err := l.Delegate.RPC("Catalog.NodeServices", &req, &out1); err != nil {
+		if err := l.Delegate.RPC(context.Background(), "Catalog.NodeServices", &req, &out1); err != nil {
 			return err
 		}
 
@@ -919,7 +1065,7 @@ func (l *State) updateSyncState() error {
 	}
 
 	var out2 structs.IndexedHealthChecks
-	if err := l.Delegate.RPC("Health.NodeChecks", &req, &out2); err != nil {
+	if err := l.Delegate.RPC(context.Background(), "Health.NodeChecks", &req, &out2); err != nil {
 		return err
 	}
 
@@ -937,6 +1083,7 @@ func (l *State) updateSyncState() error {
 	// Check if node info needs syncing
 	if svcNode == nil || svcNode.ID != l.config.NodeID ||
 		!reflect.DeepEqual(svcNode.TaggedAddresses, l.config.TaggedAddresses) ||
+		!reflect.DeepEqual(svcNode.Locality, l.config.NodeLocality) ||
 		!reflect.DeepEqual(svcNode.Meta, l.metadata) {
 		l.nodeInfoInSync = false
 	}
@@ -973,12 +1120,39 @@ func (l *State) updateSyncState() error {
 			continue
 		}
 
+		// Make a shallow copy since we may mutate it below and other readers
+		// may be reading it and we want to avoid a race.
+		nextService := *ls.Service
+		changed := false
+
 		// If our definition is different, we need to update it. Make a
 		// copy so that we don't retain a pointer to any actual state
 		// store info for in-memory RPCs.
-		if ls.Service.EnableTagOverride {
-			ls.Service.Tags = make([]string, len(rs.Tags))
-			copy(ls.Service.Tags, rs.Tags)
+		if nextService.EnableTagOverride {
+			nextService.Tags = stringslice.CloneStringSlice(rs.Tags)
+			changed = true
+		}
+
+		// Merge any tagged addresses with the consul- prefix (set by the server)
+		// back into the local state.
+		if !reflect.DeepEqual(nextService.TaggedAddresses, rs.TaggedAddresses) {
+			// Make a copy of TaggedAddresses to prevent races when writing
+			// since other goroutines may be reading from the map
+			m := make(map[string]structs.ServiceAddress)
+			for k, v := range nextService.TaggedAddresses {
+				m[k] = v
+			}
+			for k, v := range rs.TaggedAddresses {
+				if strings.HasPrefix(k, structs.MetaKeyReservedPrefix) {
+					m[k] = v
+				}
+			}
+			nextService.TaggedAddresses = m
+			changed = true
+		}
+
+		if changed {
+			ls.Service = &nextService
 		}
 		ls.InSync = ls.Service.IsSame(rs)
 	}
@@ -1073,6 +1247,9 @@ func (l *State) SyncChanges() error {
 	defer l.Unlock()
 
 	// Sync the node level info if we need to.
+	// At the start to guarantee sync even if services or checks fail,
+	// which is more likely because there are more syncs happening for them.
+
 	if l.nodeInfoInSync {
 		l.logger.Debug("Node info in sync")
 	} else {
@@ -1081,10 +1258,7 @@ func (l *State) SyncChanges() error {
 		}
 	}
 
-	// We will do node-level info syncing at the end, since it will get
-	// updated by a service or check sync anyway, given how the register
-	// API works.
-
+	var errs error
 	// Sync the services
 	// (logging happens in the helper methods)
 	for id, s := range l.services {
@@ -1098,7 +1272,7 @@ func (l *State) SyncChanges() error {
 			l.logger.Debug("Service in sync", "service", id.String())
 		}
 		if err != nil {
-			return err
+			errs = multierror.Append(errs, err)
 		}
 	}
 
@@ -1119,10 +1293,10 @@ func (l *State) SyncChanges() error {
 			l.logger.Debug("Check in sync", "check", id.String())
 		}
 		if err != nil {
-			return err
+			errs = multierror.Append(errs, err)
 		}
 	}
-	return nil
+	return errs
 }
 
 // deleteService is used to delete a service from the server
@@ -1131,7 +1305,12 @@ func (l *State) deleteService(key structs.ServiceID) error {
 		return fmt.Errorf("ServiceID missing")
 	}
 
-	st := l.aclTokenForServiceSync(key, l.tokens.AgentToken)
+	// Always use the agent token to delete without trying the service token.
+	// This works because the agent token really must have node:write
+	// permission and node:write allows deregistration of services/checks on
+	// that node. Because the service token may have been deleted, using the
+	// agent token without fallback logic is a bit faster, simpler, and safer.
+	st := l.tokens.AgentToken()
 	req := structs.DeregisterRequest{
 		Datacenter:     l.config.Datacenter,
 		Node:           l.config.NodeName,
@@ -1140,7 +1319,7 @@ func (l *State) deleteService(key structs.ServiceID) error {
 		WriteRequest:   structs.WriteRequest{Token: st},
 	}
 	var out struct{}
-	err := l.Delegate.RPC("Catalog.Deregister", &req, &out)
+	err := l.Delegate.RPC(context.Background(), "Catalog.Deregister", &req, &out)
 	switch {
 	case err == nil || strings.Contains(err.Error(), "Unknown service"):
 		delete(l.services, key)
@@ -1161,7 +1340,9 @@ func (l *State) deleteService(key structs.ServiceID) error {
 		// todo(fs): some backoff strategy might be a better solution
 		l.services[key].InSync = true
 		accessorID := l.aclAccessorID(st)
-		l.logger.Warn("Service deregistration blocked by ACLs", "service", key.String(), "accessorID", accessorID)
+		l.logger.Warn("Service deregistration blocked by ACLs",
+			"service", key.String(),
+			"accessorID", acl.AliasIfAnonymousToken(accessorID))
 		metrics.IncrCounter([]string{"acl", "blocked", "service", "deregistration"}, 1)
 		return nil
 
@@ -1180,7 +1361,9 @@ func (l *State) deleteCheck(key structs.CheckID) error {
 		return fmt.Errorf("CheckID missing")
 	}
 
-	ct := l.aclTokenForCheckSync(key, l.tokens.AgentToken)
+	// Always use the agent token for deletion. Refer to deleteService() for
+	// an explanation.
+	ct := l.tokens.AgentToken()
 	req := structs.DeregisterRequest{
 		Datacenter:     l.config.Datacenter,
 		Node:           l.config.NodeName,
@@ -1189,7 +1372,7 @@ func (l *State) deleteCheck(key structs.CheckID) error {
 		WriteRequest:   structs.WriteRequest{Token: ct},
 	}
 	var out struct{}
-	err := l.Delegate.RPC("Catalog.Deregister", &req, &out)
+	err := l.Delegate.RPC(context.Background(), "Catalog.Deregister", &req, &out)
 	switch {
 	case err == nil || strings.Contains(err.Error(), "Unknown check"):
 		l.pruneCheck(key)
@@ -1201,7 +1384,9 @@ func (l *State) deleteCheck(key structs.CheckID) error {
 		// todo(fs): some backoff strategy might be a better solution
 		l.checks[key].InSync = true
 		accessorID := l.aclAccessorID(ct)
-		l.logger.Warn("Check deregistration blocked by ACLs", "check", key.String(), "accessorID", accessorID)
+		l.logger.Warn("Check deregistration blocked by ACLs",
+			"check", key.String(),
+			"accessorID", acl.AliasIfAnonymousToken(accessorID))
 		metrics.IncrCounter([]string{"acl", "blocked", "check", "deregistration"}, 1)
 		return nil
 
@@ -1222,9 +1407,32 @@ func (l *State) pruneCheck(id structs.CheckID) {
 	delete(l.checks, id)
 }
 
+// serviceRegistrationTokenFallback returns a fallback function to be used when
+// determining the token to use for service sync.
+//
+// The fallback function will return the config file registration token if the
+// given service was sourced from a service definition in a config file.
+func (l *State) serviceRegistrationTokenFallback(key structs.ServiceID) func() string {
+	return func() string {
+		if s := l.services[key]; s != nil && s.IsLocallyDefined {
+			return l.tokens.ConfigFileRegistrationToken()
+		}
+		return ""
+	}
+}
+
+func (l *State) checkRegistrationTokenFallback(key structs.CheckID) func() string {
+	return func() string {
+		if s := l.checks[key]; s != nil && s.IsLocallyDefined {
+			return l.tokens.ConfigFileRegistrationToken()
+		}
+		return ""
+	}
+}
+
 // syncService is used to sync a service to the server
 func (l *State) syncService(key structs.ServiceID) error {
-	st := l.aclTokenForServiceSync(key, l.tokens.UserToken)
+	st := l.aclTokenForServiceSync(key, l.serviceRegistrationTokenFallback(key), l.tokens.UserToken)
 
 	// If the service has associated checks that are out of sync,
 	// piggyback them on the service sync so they are part of the
@@ -1240,7 +1448,7 @@ func (l *State) syncService(key structs.ServiceID) error {
 		if !key.Matches(c.Check.CompoundServiceID()) {
 			continue
 		}
-		if st != l.aclTokenForCheckSync(checkKey, l.tokens.UserToken) {
+		if st != l.aclTokenForCheckSync(checkKey, l.checkRegistrationTokenFallback(checkKey), l.tokens.UserToken) {
 			continue
 		}
 		checks = append(checks, c.Check)
@@ -1267,7 +1475,7 @@ func (l *State) syncService(key structs.ServiceID) error {
 	}
 
 	var out struct{}
-	err := l.Delegate.RPC("Catalog.Register", &req, &out)
+	err := l.Delegate.RPC(context.Background(), "Catalog.Register", &req, &out)
 	switch {
 	case err == nil:
 		l.services[key].InSync = true
@@ -1290,7 +1498,9 @@ func (l *State) syncService(key structs.ServiceID) error {
 			l.checks[checkKey].InSync = true
 		}
 		accessorID := l.aclAccessorID(st)
-		l.logger.Warn("Service registration blocked by ACLs", "service", key.String(), "accessorID", accessorID)
+		l.logger.Warn("Service registration blocked by ACLs",
+			"service", key.String(),
+			"accessorID", acl.AliasIfAnonymousToken(accessorID))
 		metrics.IncrCounter([]string{"acl", "blocked", "service", "registration"}, 1)
 		return nil
 
@@ -1306,7 +1516,7 @@ func (l *State) syncService(key structs.ServiceID) error {
 // syncCheck is used to sync a check to the server
 func (l *State) syncCheck(key structs.CheckID) error {
 	c := l.checks[key]
-	ct := l.aclTokenForCheckSync(key, l.tokens.UserToken)
+	ct := l.aclTokenForCheckSync(key, l.checkRegistrationTokenFallback(key), l.tokens.UserToken)
 	req := structs.RegisterRequest{
 		Datacenter:      l.config.Datacenter,
 		ID:              l.config.NodeID,
@@ -1329,7 +1539,7 @@ func (l *State) syncCheck(key structs.CheckID) error {
 	}
 
 	var out struct{}
-	err := l.Delegate.RPC("Catalog.Register", &req, &out)
+	err := l.Delegate.RPC(context.Background(), "Catalog.Register", &req, &out)
 	switch {
 	case err == nil:
 		l.checks[key].InSync = true
@@ -1344,7 +1554,9 @@ func (l *State) syncCheck(key structs.CheckID) error {
 		// todo(fs): some backoff strategy might be a better solution
 		l.checks[key].InSync = true
 		accessorID := l.aclAccessorID(ct)
-		l.logger.Warn("Check registration blocked by ACLs", "check", key.String(), "accessorID", accessorID)
+		l.logger.Warn("Check registration blocked by ACLs",
+			"check", key.String(),
+			"accessorID", acl.AliasIfAnonymousToken(accessorID))
 		metrics.IncrCounter([]string{"acl", "blocked", "check", "registration"}, 1)
 		return nil
 
@@ -1365,11 +1577,13 @@ func (l *State) syncNodeInfo() error {
 		Node:            l.config.NodeName,
 		Address:         l.config.AdvertiseAddr,
 		TaggedAddresses: l.config.TaggedAddresses,
+		Locality:        l.config.NodeLocality,
 		NodeMeta:        l.metadata,
+		EnterpriseMeta:  l.agentEnterpriseMeta,
 		WriteRequest:    structs.WriteRequest{Token: at},
 	}
 	var out struct{}
-	err := l.Delegate.RPC("Catalog.Register", &req, &out)
+	err := l.Delegate.RPC(context.Background(), "Catalog.Register", &req, &out)
 	switch {
 	case err == nil:
 		l.nodeInfoInSync = true
@@ -1381,7 +1595,9 @@ func (l *State) syncNodeInfo() error {
 		// todo(fs): some backoff strategy might be a better solution
 		l.nodeInfoInSync = true
 		accessorID := l.aclAccessorID(at)
-		l.logger.Warn("Node info update blocked by ACLs", "node", l.config.NodeID, "accessorID", accessorID)
+		l.logger.Warn("Node info update blocked by ACLs",
+			"node", l.config.NodeID,
+			"accessorID", acl.AliasIfAnonymousToken(accessorID))
 		metrics.IncrCounter([]string{"acl", "blocked", "node", "registration"}, 1)
 		return nil
 
@@ -1411,7 +1627,7 @@ func (l *State) notifyIfAliased(serviceID structs.ServiceID) {
 // critical purposes, such as logging. Therefore we interpret all errors as empty-string
 // so we can safely log it without handling non-critical errors at the usage site.
 func (l *State) aclAccessorID(secretID string) string {
-	ident, err := l.Delegate.ResolveTokenToIdentity(secretID)
+	ident, err := l.Delegate.ResolveTokenAndDefaultMeta(secretID, nil, nil)
 	if acl.IsErrNotFound(err) {
 		return ""
 	}
@@ -1419,8 +1635,23 @@ func (l *State) aclAccessorID(secretID string) string {
 		l.logger.Debug("non-critical error resolving acl token accessor for logging", "error", err)
 		return ""
 	}
-	if ident == nil {
-		return ""
+	return ident.AccessorID()
+}
+
+func cloneService(ns *structs.NodeService) (*structs.NodeService, error) {
+	// TODO: consider doing a hand-managed clone function
+	raw, err := copystructure.Copy(ns)
+	if err != nil {
+		return nil, err
 	}
-	return ident.ID()
+	return raw.(*structs.NodeService), err
+}
+
+func cloneCheck(check *structs.HealthCheck) (*structs.HealthCheck, error) {
+	// TODO: consider doing a hand-managed clone function
+	raw, err := copystructure.Copy(check)
+	if err != nil {
+		return nil, err
+	}
+	return raw.(*structs.HealthCheck), err
 }
