@@ -10,12 +10,12 @@ import (
 	"time"
 
 	"github.com/google/go-cmp/cmp"
-	"github.com/hashicorp/hcl"
 	"github.com/mitchellh/copystructure"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 
 	"github.com/hashicorp/consul-net-rpc/go-msgpack/codec"
+	"github.com/hashicorp/hcl"
 
 	"github.com/hashicorp/consul/acl"
 	"github.com/hashicorp/consul/agent/cache"
@@ -1910,6 +1910,10 @@ func TestDecodeConfigEntry(t *testing.T) {
 							  name   = "hdr-suffix"
 							  suffix = "suffix"
 							},
+							{
+							  name   = "hdr-contains"
+							  contains = "contains"
+							},
 							{
 							  name  = "hdr-regex"
 							  regex = "regex"
@@ -1918,7 +1922,12 @@ func TestDecodeConfigEntry(t *testing.T) {
 							  name    = "hdr-absent"
 							  present = true
 							  invert  = true
-							}
+							},
+							{
+							  name  = "hdr-ignore-case"
+							  exact = "exact"
+							  ignore_case = true
+							},
 						  ]
 						}
 					  },
@@ -1987,6 +1996,10 @@ func TestDecodeConfigEntry(t *testing.T) {
 							  Name   = "hdr-suffix"
 							  Suffix = "suffix"
 							},
+							{
+							  Name   = "hdr-contains"
+							  Contains = "contains"
+							},
 							{
 							  Name  = "hdr-regex"
 							  Regex = "regex"
@@ -1995,6 +2008,11 @@ func TestDecodeConfigEntry(t *testing.T) {
 							  Name    = "hdr-absent"
 							  Present = true
 							  Invert  = true
+							},
+							{
+							  Name  = "hdr-ignore-case"
+							  Exact = "exact"
+							  IgnoreCase = true
 							}
 						  ]
 						}
@@ -2064,6 +2082,10 @@ func TestDecodeConfigEntry(t *testing.T) {
 											Name:   "hdr-suffix",
 											Suffix: "suffix",
 										},
+										{
+											Name:     "hdr-contains",
+											Contains: "contains",
+										},
 										{
 											Name:  "hdr-regex",
 											Regex: "regex",
@@ -2073,6 +2095,11 @@ func TestDecodeConfigEntry(t *testing.T) {
 											Present: true,
 											Invert:  true,
 										},
+										{
+											Name:       "hdr-ignore-case",
+											Exact:      "exact",
+											IgnoreCase: true,
+										},
 									},
 								},
 							},
@@ -2134,7 +2161,7 @@ func TestDecodeConfigEntry(t *testing.T) {
 			},
 		},
 		{
-			name: "mesh",
+			name: "mesh: kitchen sink",
 			snake: `
 				kind = "mesh"
 				meta {
@@ -2145,6 +2172,7 @@ func TestDecodeConfigEntry(t *testing.T) {
 					mesh_destinations_only = true
 				}
 				allow_enabling_permissive_mutual_tls = true
+                validate_clusters = true
 				tls {
 					incoming {
 						tls_min_version = "TLSv1_1"
@@ -2163,9 +2191,17 @@ func TestDecodeConfigEntry(t *testing.T) {
 						]
 					}
 				}
-				http {
-					sanitize_x_forwarded_client_cert = true
-				}
+                http {
+                    sanitize_x_forwarded_client_cert = true
+                    incoming {
+                        request_normalization {
+                        	insecure_disable_path_normalization = true
+                            merge_slashes = true
+                            path_with_escaped_slashes_action = "UNESCAPE_AND_FORWARD"
+							headers_with_underscores_action = "DROP_HEADER"
+                        }
+                    }
+                }
 				peering {
 					peer_through_mesh_gateways = true
 				}
@@ -2180,6 +2216,7 @@ func TestDecodeConfigEntry(t *testing.T) {
 					MeshDestinationsOnly = true
 				}
 				AllowEnablingPermissiveMutualTLS = true
+                ValidateClusters = true
 				TLS {
 					Incoming {
 						TLSMinVersion = "TLSv1_1"
@@ -2198,9 +2235,17 @@ func TestDecodeConfigEntry(t *testing.T) {
 						]
 					}
 				}
-				HTTP {
-					SanitizeXForwardedClientCert = true
-				}
+                HTTP {
+                    SanitizeXForwardedClientCert = true
+                    Incoming {
+                        RequestNormalization {
+                        	InsecureDisablePathNormalization = true
+                            MergeSlashes = true
+                            PathWithEscapedSlashesAction = "UNESCAPE_AND_FORWARD"
+							HeadersWithUnderscoresAction = "DROP_HEADER"
+                        }
+                    }
+                }
 				Peering {
 					PeerThroughMeshGateways = true
 				}
@@ -2214,6 +2259,7 @@ func TestDecodeConfigEntry(t *testing.T) {
 					MeshDestinationsOnly: true,
 				},
 				AllowEnablingPermissiveMutualTLS: true,
+				ValidateClusters:                 true,
 				TLS: &MeshTLSConfig{
 					Incoming: &MeshDirectionalTLSConfig{
 						TLSMinVersion: types.TLSv1_1,
@@ -2234,6 +2280,14 @@ func TestDecodeConfigEntry(t *testing.T) {
 				},
 				HTTP: &MeshHTTPConfig{
 					SanitizeXForwardedClientCert: true,
+					Incoming: &MeshDirectionalHTTPConfig{
+						RequestNormalization: &RequestNormalizationMeshConfig{
+							InsecureDisablePathNormalization: true, // note: this is the opposite of the recommended default
+							MergeSlashes:                     true,
+							PathWithEscapedSlashesAction:     "UNESCAPE_AND_FORWARD",
+							HeadersWithUnderscoresAction:     "DROP_HEADER",
+						},
+					},
 				},
 				Peering: &PeeringMeshConfig{
 					PeerThroughMeshGateways: true,
@@ -3225,6 +3279,14 @@ func TestServiceConfigEntry(t *testing.T) {
 			},
 			validateErr: `Invalid MutualTLSMode "invalid-mtls-mode". Must be one of "", "strict", or "permissive".`,
 		},
+		"validate: invalid Protocol in service-defaults": {
+			entry: &ServiceConfigEntry{
+				Kind:     ServiceDefaults,
+				Name:     "web",
+				Protocol: "blah",
+			},
+			validateErr: `invalid value for protocol: blah`,
+		},
 	}
 	testConfigEntryNormalizeAndValidate(t, cases)
 }
