@@ -1,3 +1,6 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: BUSL-1.1
+
 package structs
 
 import (
@@ -8,13 +11,18 @@ import (
 func TestServiceDefinition(t testing.T) *ServiceDefinition {
 	return &ServiceDefinition{
 		Name: "db",
+		Port: 1234,
 	}
 }
 
 // TestServiceDefinitionProxy returns a ServiceDefinition for a proxy.
 func TestServiceDefinitionProxy(t testing.T) *ServiceDefinition {
 	return &ServiceDefinition{
-		Kind:             ServiceKindConnectProxy,
-		ProxyDestination: "db",
+		Kind: ServiceKindConnectProxy,
+		Name: "foo-proxy",
+		Port: 1234,
+		Proxy: &ConnectProxyConfig{
+			DestinationServiceName: "db",
+		},
 	}
 }
