@@ -18,27 +18,24 @@ import (
 	envoy_config_core_v3 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
 	envoy_listener_v3 "github.com/envoyproxy/go-control-plane/envoy/config/listener/v3"
 	envoy_discovery_v3 "github.com/envoyproxy/go-control-plane/envoy/service/discovery/v3"
-	"github.com/hashicorp/go-hclog"
-	goversion "github.com/hashicorp/go-version"
 	"google.golang.org/grpc/codes"
 	"google.golang.org/grpc/status"
 	"google.golang.org/protobuf/proto"
 	"google.golang.org/protobuf/types/known/anypb"
 
+	"github.com/hashicorp/go-hclog"
+	goversion "github.com/hashicorp/go-version"
+
 	"github.com/hashicorp/consul/agent/envoyextensions"
 	external "github.com/hashicorp/consul/agent/grpc-external"
 	"github.com/hashicorp/consul/agent/grpc-external/limiter"
 	"github.com/hashicorp/consul/agent/proxycfg"
+	"github.com/hashicorp/consul/agent/structs"
 	"github.com/hashicorp/consul/agent/xds/configfetcher"
 	"github.com/hashicorp/consul/agent/xds/extensionruntime"
-	"github.com/hashicorp/consul/agent/xdsv2"
 	"github.com/hashicorp/consul/envoyextensions/extensioncommon"
 	"github.com/hashicorp/consul/envoyextensions/xdscommon"
-	proxysnapshot "github.com/hashicorp/consul/internal/mesh/proxy-snapshot"
-	proxytracker "github.com/hashicorp/consul/internal/mesh/proxy-tracker"
 	"github.com/hashicorp/consul/logging"
-	pbmesh "github.com/hashicorp/consul/proto-public/pbmesh/v2beta1"
-	"github.com/hashicorp/consul/proto-public/pbresource"
 	"github.com/hashicorp/consul/version"
 )
 
@@ -99,36 +96,14 @@ func (s *Server) DeltaAggregatedResources(stream ADSDeltaStream) error {
 }
 
 // getEnvoyConfiguration is a utility function that instantiates the proper
-// Envoy resource generator based on whether it was passed a ConfigSource or
-// ProxyState implementation of the ProxySnapshot interface and returns the
-// generated Envoy configuration.
-func getEnvoyConfiguration(proxySnapshot proxysnapshot.ProxySnapshot, logger hclog.Logger, cfgFetcher configfetcher.ConfigFetcher) (map[string][]proto.Message, error) {
-	switch proxySnapshot.(type) {
-	case *proxycfg.ConfigSnapshot:
-		logger.Trace("ProxySnapshot update channel received a ProxySnapshot of type ConfigSnapshot")
-		generator := NewResourceGenerator(
-			logger,
-			cfgFetcher,
-			true,
-		)
-
-		c := proxySnapshot.(*proxycfg.ConfigSnapshot)
-		return generator.AllResourcesFromSnapshot(c)
-	case *proxytracker.ProxyState:
-		logger.Trace("ProxySnapshot update channel received a ProxySnapshot of type ProxyState")
-		generator := xdsv2.NewResourceGenerator(
-			logger,
-		)
-		c := proxySnapshot.(*proxytracker.ProxyState)
-		resources, err := generator.AllResourcesFromIR(c)
-		if err != nil {
-			logger.Error("error generating resources from proxy state template", "err", err)
-			return nil, err
-		}
-		return resources, nil
-	default:
-		return nil, errors.New("proxysnapshot must be of type ProxyState or ConfigSnapshot")
-	}
+// Envoy resource generator and returns the generated Envoy configuration.
+func getEnvoyConfiguration(snapshot *proxycfg.ConfigSnapshot, logger hclog.Logger, cfgFetcher configfetcher.ConfigFetcher) (map[string][]proto.Message, error) {
+	generator := NewResourceGenerator(
+		logger,
+		cfgFetcher,
+		true,
+	)
+	return generator.AllResourcesFromSnapshot(snapshot)
 }
 
 const (
@@ -145,9 +120,9 @@ func (s *Server) processDelta(stream ADSDeltaStream, reqCh <-chan *envoy_discove
 
 	// Loop state
 	var (
-		proxySnapshot    proxysnapshot.ProxySnapshot
+		snapshot         *proxycfg.ConfigSnapshot
 		node             *envoy_config_core_v3.Node
-		stateCh          <-chan proxysnapshot.ProxySnapshot
+		stateCh          <-chan *proxycfg.ConfigSnapshot
 		drainCh          limiter.SessionTerminatedChan
 		cfgSrcTerminated proxycfg.SrcTerminatedChan
 		watchCancel      func()
@@ -178,13 +153,13 @@ func (s *Server) processDelta(stream ADSDeltaStream, reqCh <-chan *envoy_discove
 	// Configure handlers for each type of request we currently care about.
 	handlers := map[string]*xDSDeltaType{
 		xdscommon.ListenerType: newDeltaType(logger, stream, xdscommon.ListenerType, func() bool {
-			return proxySnapshot.AllowEmptyListeners()
+			return snapshot.AllowEmptyListeners()
 		}),
 		xdscommon.RouteType: newDeltaType(logger, stream, xdscommon.RouteType, func() bool {
-			return proxySnapshot.AllowEmptyRoutes()
+			return snapshot.AllowEmptyRoutes()
 		}),
 		xdscommon.ClusterType: newDeltaType(logger, stream, xdscommon.ClusterType, func() bool {
-			return proxySnapshot.AllowEmptyClusters()
+			return snapshot.AllowEmptyClusters()
 		}),
 		xdscommon.EndpointType: newDeltaType(logger, stream, xdscommon.EndpointType, nil),
 		xdscommon.SecretType:   newDeltaType(logger, stream, xdscommon.SecretType, nil), // TODO allowEmptyFn
@@ -213,8 +188,8 @@ func (s *Server) processDelta(stream ADSDeltaStream, reqCh <-chan *envoy_discove
 		authTimer = time.After(s.AuthCheckFrequency)
 	}
 
-	checkStreamACLs := func(proxySnap proxysnapshot.ProxySnapshot) error {
-		return s.authorize(stream.Context(), proxySnap)
+	checkStreamACLs := func(snapshot *proxycfg.ConfigSnapshot) error {
+		return s.authorize(stream.Context(), snapshot)
 	}
 
 	for {
@@ -225,7 +200,7 @@ func (s *Server) processDelta(stream ADSDeltaStream, reqCh <-chan *envoy_discove
 			return errOverwhelmed
 		case <-authTimer:
 			// It's been too long since a Discovery{Request,Response} so recheck ACLs.
-			if err := checkStreamACLs(proxySnapshot); err != nil {
+			if err := checkStreamACLs(snapshot); err != nil {
 				return err
 			}
 			extendAuthTimer()
@@ -280,9 +255,9 @@ func (s *Server) processDelta(stream ADSDeltaStream, reqCh <-chan *envoy_discove
 				// would've already exited this loop.
 				return status.Error(codes.Aborted, "xDS stream terminated due to an irrecoverable error, please try again")
 			}
-			proxySnapshot = cs
+			snapshot = cs
 
-			newRes, err := getEnvoyConfiguration(proxySnapshot, logger, s.CfgFetcher)
+			newRes, err := getEnvoyConfiguration(snapshot, logger, s.CfgFetcher)
 			if err != nil {
 				return status.Errorf(codes.Unavailable, "failed to generate all xDS resources from the snapshot: %v", err)
 			}
@@ -294,7 +269,7 @@ func (s *Server) processDelta(stream ADSDeltaStream, reqCh <-chan *envoy_discove
 				s.ResourceMapMutateFn(newResourceMap)
 			}
 
-			if newResourceMap, err = s.applyEnvoyExtensions(newResourceMap, proxySnapshot, node); err != nil {
+			if newResourceMap, err = s.applyEnvoyExtensions(newResourceMap, snapshot, node); err != nil {
 				// err is already the result of calling status.Errorf
 				return err
 			}
@@ -334,7 +309,7 @@ func (s *Server) processDelta(stream ADSDeltaStream, reqCh <-chan *envoy_discove
 			}
 
 			// Start authentication process, we need the proxyID
-			proxyID := newResourceIDFromEnvoyNode(node)
+			proxyID := structs.NewServiceID(node.Id, parseEnterpriseMeta(node))
 
 			// Start watching config for that proxy
 			var err error
@@ -357,14 +332,14 @@ func (s *Server) processDelta(stream ADSDeltaStream, reqCh <-chan *envoy_discove
 			// state machine.
 			defer watchCancel()
 
-			logger = logger.With("service_id", proxyID.Name) // enhance future logs
+			logger = logger.With("service_id", proxyID.String()) // enhance future logs
 
 			logger.Trace("watching proxy, pending initial proxycfg snapshot for xDS")
 
 			// Now wait for the config so we can check ACL
 			state = stateDeltaPendingInitialConfig
 		case stateDeltaPendingInitialConfig:
-			if proxySnapshot == nil {
+			if snapshot == nil {
 				// Nothing we can do until we get the initial config
 				continue
 			}
@@ -373,7 +348,7 @@ func (s *Server) processDelta(stream ADSDeltaStream, reqCh <-chan *envoy_discove
 			state = stateDeltaRunning
 
 			// Upgrade the logger
-			loggerName := proxySnapshot.LoggerName()
+			loggerName := snapshot.LoggerName()
 			if loggerName != "" {
 				logger = logger.Named(loggerName)
 			}
@@ -384,7 +359,7 @@ func (s *Server) processDelta(stream ADSDeltaStream, reqCh <-chan *envoy_discove
 			fallthrough
 		case stateDeltaRunning:
 			// Check ACLs on every Discovery{Request,Response}.
-			if err := checkStreamACLs(proxySnapshot); err != nil {
+			if err := checkStreamACLs(snapshot); err != nil {
 				return err
 			}
 			// For the first time through the state machine, this is when the
@@ -433,37 +408,7 @@ func (s *Server) processDelta(stream ADSDeltaStream, reqCh <-chan *envoy_discove
 	}
 }
 
-// newResourceIDFromEnvoyNode is a utility function that allows creating a
-// Resource ID from an Envoy proxy node so that existing delta calls can easily
-// use ProxyWatcher interface arguments for Watch().
-func newResourceIDFromEnvoyNode(node *envoy_config_core_v3.Node) *pbresource.ID {
-	entMeta := parseEnterpriseMeta(node)
-
-	return &pbresource.ID{
-		Name: node.Id,
-		Tenancy: &pbresource.Tenancy{
-			Namespace: entMeta.NamespaceOrDefault(),
-			Partition: entMeta.PartitionOrDefault(),
-		},
-		Type: pbmesh.ProxyStateTemplateType,
-	}
-}
-
-func (s *Server) applyEnvoyExtensions(resources *xdscommon.IndexedResources, proxySnapshot proxysnapshot.ProxySnapshot, node *envoy_config_core_v3.Node) (*xdscommon.IndexedResources, error) {
-	// TODO(proxystate)
-	// This is a workaround for now as envoy extensions are not yet supported with ProxyState.
-	// For now, we cast to proxycfg.ConfigSnapshot and no-op if it's the pbmesh.ProxyState type.
-	var snapshot *proxycfg.ConfigSnapshot
-	switch proxySnapshot.(type) {
-	//TODO(proxystate): implement envoy extensions for ProxyState
-	case *proxytracker.ProxyState:
-		return resources, nil
-	case *proxycfg.ConfigSnapshot:
-		snapshot = proxySnapshot.(*proxycfg.ConfigSnapshot)
-	default:
-		return nil, status.Errorf(codes.InvalidArgument,
-			"unsupported config snapshot type to apply envoy extensions to %T", proxySnapshot)
-	}
+func (s *Server) applyEnvoyExtensions(resources *xdscommon.IndexedResources, snapshot *proxycfg.ConfigSnapshot, node *envoy_config_core_v3.Node) (*xdscommon.IndexedResources, error) {
 	var err error
 	envoyVersion := xdscommon.DetermineEnvoyVersionFromNode(node)
 	consulVersion, err := goversion.NewVersion(version.Version)
