@@ -1,3 +1,8 @@
+/**
+ * Copyright (c) HashiCorp, Inc.
+ * SPDX-License-Identifier: BUSL-1.1
+ */
+
 /* globals CodeMirror */
 export function initialize(application) {
   const appName = application.application.name;
@@ -8,7 +13,7 @@ export function initialize(application) {
   );
   // configure syntax highlighting for CodeMirror
   CodeMirror.modeURL = {
-    replace: function(n, mode) {
+    replace: function (n, mode) {
       switch (mode.trim()) {
         case 'javascript':
           return fs.get(['codemirror', 'mode', 'javascript', 'javascript.js'].join('/'));
@@ -16,6 +21,8 @@ export function initialize(application) {
           return fs.get(['codemirror', 'mode', 'ruby', 'ruby.js'].join('/'));
         case 'yaml':
           return fs.get(['codemirror', 'mode', 'yaml', 'yaml.js'].join('/'));
+        case 'xml':
+          return fs.get(['codemirror', 'mode', 'xml', 'xml.js'].join('/'));
       }
     },
   };
