@@ -4,7 +4,6 @@
  */
 
 import Service, { inject as service } from '@ember/service';
-import lint from 'consul-ui/utils/editor/lint';
 const MODES = [
   {
     name: 'JSON',
@@ -41,10 +40,6 @@ export default class LinterService extends Service {
     return MODES;
   }
 
-  lint() {
-    return lint(...arguments);
-  }
-
   getEditor(element) {
     return this.dom.element('textarea + div', element).CodeMirror;
   }
