@@ -1,11 +1,12 @@
+/**
+ * Copyright (c) HashiCorp, Inc.
+ * SPDX-License-Identifier: BUSL-1.1
+ */
+
 import RepositoryService from 'consul-ui/services/repository';
-import statusFactory from 'consul-ui/utils/acls-status';
-import isValidServerErrorFactory from 'consul-ui/utils/http/acl/is-valid-server-error';
 import { PRIMARY_KEY, SLUG_KEY } from 'consul-ui/models/binding-rule';
 import dataSource from 'consul-ui/decorators/data-source';
 
-const isValidServerError = isValidServerErrorFactory();
-const status = statusFactory(isValidServerError, Promise);
 const MODEL_NAME = 'binding-rule';
 
 export default class BindingRuleService extends RepositoryService {
@@ -21,12 +22,8 @@ export default class BindingRuleService extends RepositoryService {
     return SLUG_KEY;
   }
 
-  @dataSource('/:ns/:dc/binding-rules')
-  async findAllByDatacenter() {
-    return super.findAllByDatacenter(...arguments);
-  }
-
-  status(obj) {
-    return status(obj);
+  @dataSource('/:partition/:ns/:dc/binding-rules/for-auth-method/:authmethod')
+  async findAllByAuthMethod() {
+    return super.findAll(...arguments);
   }
 }
